@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2005, 2015, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2005, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -38,6 +38,8 @@
 #include <my_getopt.h>
 #include <m_string.h>
 #include <welcome_copyright_notice.h>	/* ORACLE_WELCOME_COPYRIGHT_NOTICE */
+#include "typelib.h"
+#include "prealloced_array.h"
 
 /* Only parts of these files are included from the InnoDB codebase.
 The parts not included are excluded by #ifndef UNIV_INNOCHECKSUM. */
@@ -253,7 +255,7 @@ open_file(
 
 	if (hFile == INVALID_HANDLE_VALUE) {
 		/* print the error message. */
-		fprintf(stderr, "Filename::%s %s\n",
+		fprintf(stderr, "Filename::%s %s\n", name,
 			error_message(GetLastError()));
 
 			return (NULL);
@@ -355,12 +357,14 @@ is_page_corrupted(
 
 		if (is_log_enabled) {
 			fprintf(log_file,
-				"page::%" PRIuMAX "; log sequence number:first = %lu;"
-				" second = %lu\n",
+				"page::%" PRIuMAX
+				"; log sequence number:first = " ULINTPF
+				"; second = " ULINTPF "\n",
 				cur_page_num, logseq, logseqfield);
 			if (logseq != logseqfield) {
 				fprintf(log_file,
-					"Fail; page %" PRIuMAX " invalid (fails log "
+					"Fail; page %" PRIuMAX
+					" invalid (fails log "
 					"sequence number check)\n",
 					cur_page_num);
 			}
@@ -634,8 +638,9 @@ parse_page(
 				"\tindex id=%llu,", cur_page_num, id);
 
 			fprintf(file,
-				" page level=%lu, No. of records=%lu,"
-				" garbage=%lu, %s\n",
+				" page level=" ULINTPF
+				", No. of records=" ULINTPF
+				", garbage=" ULINTPF ", %s\n",
 				page_header_get_field(page, PAGE_LEVEL),
 				page_header_get_field(page, PAGE_N_RECS),
 				page_header_get_field(page, PAGE_GARBAGE), str);
@@ -989,8 +994,8 @@ static void usage(void)
 extern "C" my_bool
 innochecksum_get_one_option(
 	int			optid,
-	const struct my_option	*opt __attribute__((unused)),
-	char			*argument __attribute__((unused)))
+	const struct my_option	*opt MY_ATTRIBUTE((unused)),
+	char			*argument MY_ATTRIBUTE((unused)))
 {
 	switch (optid) {
 #ifndef DBUG_OFF
@@ -1162,7 +1167,7 @@ int main(
 	/* our input filename. */
 	char*		filename;
 	/* Buffer to store pages read. */
-	byte*		buf = NULL;
+	Prealloced_array<byte, 1> buf(PSI_NOT_INSTRUMENTED);
 	/* bytes read count */
 	ulong		bytes;
 	/* Buffer to decompress page.*/
@@ -1242,8 +1247,8 @@ int main(
 	}
 
 
-	buf = (byte*) malloc(UNIV_PAGE_SIZE_MAX * 2);
-	tbuf = buf + UNIV_PAGE_SIZE_MAX;
+	buf.reserve(UNIV_PAGE_SIZE_MAX * 2);
+	tbuf = buf.begin() + UNIV_PAGE_SIZE_MAX;
 
 	/* The file name is not optional. */
 	for (int i = 0; i < argc; ++i) {
@@ -1313,7 +1318,7 @@ int main(
 #endif /* _WIN32 */
 
 		/* Read the minimum page size. */
-		bytes = ulong(fread(buf, 1, UNIV_ZIP_SIZE_MIN, fil_in));
+		bytes = ulong(fread(buf.begin(), 1, UNIV_ZIP_SIZE_MIN, fil_in));
 		partial_page_read = true;
 
 		if (bytes != UNIV_ZIP_SIZE_MIN) {
@@ -1322,32 +1327,31 @@ int main(
 			fprintf(stderr, "of %d bytes.  Bytes read was %lu\n",
 				UNIV_ZIP_SIZE_MIN, bytes);
 
-			free(buf);
 			DBUG_RETURN(1);
 		}
 
 		/* enable variable is_system_tablespace when space_id of given
 		file is zero. Use to skip the checksum verification and rewrite
 		for doublewrite pages. */
-		is_system_tablespace = (!memcmp(&space_id, buf +
+		is_system_tablespace = (!memcmp(&space_id, buf.begin() +
 					FIL_PAGE_ARCH_LOG_NO_OR_SPACE_ID, 4))
 					? true : false;
 
-		const page_size_t&	page_size = get_page_size(buf);
+		const page_size_t&	page_size = get_page_size(buf.begin());
 
 		pages = (ulint) (size / page_size.physical());
 
 		if (just_count) {
 			if (read_from_stdin) {
-				fprintf(stderr, "Number of pages:%lu\n", pages);
+				fprintf(stderr, "Number of pages:" ULINTPF "\n", pages);
 			} else {
-				printf("Number of pages:%lu\n", pages);
+				printf("Number of pages:" ULINTPF "\n", pages);
 			}
 			continue;
 		} else if (verbose && !read_from_stdin) {
 			if (is_log_enabled) {
 				fprintf(log_file, "file %s = %llu bytes "
-					"(%lu pages)\n", filename, size, pages);
+					"(" ULINTPF " pages)\n", filename, size, pages);
 				if (do_one_page) {
 					fprintf(log_file, "Innochecksum: "
 						"checking page %" PRIuMAX "\n",
@@ -1382,7 +1386,6 @@ int main(
 					perror("Error: Unable to seek to "
 						"necessary offset");
 
-					free(buf);
 					DBUG_RETURN(1);
 				}
 				/* Save the current file pointer in
@@ -1390,7 +1393,6 @@ int main(
 				if (0 != fgetpos(fil_in, &pos)) {
 					perror("fgetpos");
 
-					free(buf);
 					DBUG_RETURN(1);
 				}
 			} else {
@@ -1408,7 +1410,7 @@ int main(
 					(fseeko() on stdin doesn't work). So
 					read only the remaining part of page,
 					if partial_page_read is enable. */
-					bytes = read_file(buf,
+					bytes = read_file(buf.begin(),
 							  partial_page_read,
 							  static_cast<ulong>(
 							  page_size.physical()),
@@ -1422,7 +1424,6 @@ int main(
 							"to seek to necessary "
 							"offset");
 
-						free(buf);
 						DBUG_RETURN(1);
 					}
 				}
@@ -1448,7 +1449,7 @@ int main(
 		lastt = 0;
 		while (!feof(fil_in)) {
 
-			bytes = read_file(buf, partial_page_read,
+			bytes = read_file(buf.begin(), partial_page_read,
 					  static_cast<ulong>(
 					  page_size.physical()), fil_in);
 			partial_page_read = false;
@@ -1458,40 +1459,37 @@ int main(
 			}
 
 			if (ferror(fil_in)) {
-				fprintf(stderr, "Error reading %lu bytes",
+				fprintf(stderr, "Error reading " ULINTPF " bytes",
 					page_size.physical());
 				perror(" ");
 
-				free(buf);
 				DBUG_RETURN(1);
 			}
 
 			if (bytes != page_size.physical()) {
 				fprintf(stderr, "Error: bytes read (%lu) "
-					"doesn't match page size (%lu)\n",
+					"doesn't match page size (" ULINTPF ")\n",
 					bytes, page_size.physical());
-				free(buf);
 				DBUG_RETURN(1);
 			}
 
 			if (display_format) {
 				/* for -f, analyze only the first page and exit */
-				display_format_info(buf);
+				display_format_info(buf.begin());
 				break;
 			}
 
 			if (is_system_tablespace) {
 				/* enable when page is double write buffer.*/
-				skip_page = is_page_doublewritebuffer(buf);
+				skip_page = is_page_doublewritebuffer(buf.begin());
 			} else {
 				skip_page = false;
 
-				if (!page_decompress(buf, tbuf, page_size)) {
+				if (!page_decompress(buf.begin(), tbuf, page_size)) {
 
 					fprintf(stderr,
 						"Page decompress failed");
 
-					free(buf);
 					DBUG_RETURN(1);
 				}
 			}
@@ -1502,7 +1500,7 @@ int main(
 				/* Checksum verification */
 				if (!skip_page) {
 					is_corrupted = is_page_corrupted(
-						buf, page_size);
+						buf.begin(), page_size);
 
 					if (is_corrupted) {
 						fprintf(stderr, "Fail: page "
@@ -1519,7 +1517,6 @@ int main(
 								"count::%" PRIuMAX "\n",
 								allow_mismatches);
 
-							free(buf);
 							DBUG_RETURN(1);
 						}
 					}
@@ -1528,11 +1525,10 @@ int main(
 
 			/* Rewrite checksum */
 			if (do_write
-			    && !write_file(filename, fil_in, buf,
+			    && !write_file(filename, fil_in, buf.begin(),
 					   page_size.is_compressed(), &pos,
 					   static_cast<ulong>(page_size.physical()))) {
 
-				free(buf);
 				DBUG_RETURN(1);
 			}
 
@@ -1542,7 +1538,7 @@ int main(
 			}
 
 			if (page_type_summary || page_type_dump) {
-				parse_page(buf, fil_page_type);
+				parse_page(buf.begin(), fil_page_type);
 			}
 
 			/* do counter increase and progress printing */
@@ -1586,6 +1582,5 @@ int main(
 		fclose(log_file);
 	}
 
-	free(buf);
 	DBUG_RETURN(0);
 }
