@@ -1,4 +1,4 @@
-/* Copyright (c) 2005, 2014, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2005, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,8 +16,13 @@
 #ifndef _my_plugin_h
 #define _my_plugin_h
 
+#ifndef MYSQL_ABI_CHECK
+#include <stddef.h>
+#include "mysql_version.h" /* MYSQL_VERSION_ID */
+#endif
+
 /*
-  On Windows, exports from DLL need to be declared
+  On Windows, exports from DLL need to be declared.
   Also, plugin needs to be declared as extern "C" because MSVC 
   unlike other compilers, uses C++ mangling for variables not only
   for functions.
@@ -50,7 +55,15 @@ class Item;
 
 typedef void * MYSQL_PLUGIN;
 
+#ifndef MYSQL_ABI_CHECK
 #include <mysql/services.h>
+#ifndef __WIN__
+#ifndef __STDC_FORMAT_MACROS
+#define __STDC_FORMAT_MACROS    /* Enable C99 printf format macros */
+#endif /* !__STDC_FORMAT_MACROS */
+#include <inttypes.h>
+#endif /* !__WIN__ */
+#endif /* !MYSQL_ABI_CHECK */
 
 #define MYSQL_XIDDATASIZE 128
 /**
@@ -73,7 +86,7 @@ typedef struct st_mysql_xid MYSQL_XID;
   Plugin API. Common for all plugin types.
 */
 
-#define MYSQL_PLUGIN_INTERFACE_VERSION 0x0104
+#define MYSQL_PLUGIN_INTERFACE_VERSION 0x0107
 
 /*
   The allowable types of plugins
@@ -87,7 +100,9 @@ typedef struct st_mysql_xid MYSQL_XID;
 #define MYSQL_REPLICATION_PLUGIN     6	/* The replication plugin type */
 #define MYSQL_AUTHENTICATION_PLUGIN  7  /* The authentication plugin type */
 #define MYSQL_VALIDATE_PASSWORD_PLUGIN  8   /* validate password plugin type */
-#define MYSQL_MAX_PLUGIN_TYPE_NUM    9  /* The number of plugin types   */
+#define MYSQL_GROUP_REPLICATION_PLUGIN  9  /* The Group Replication plugin */
+#define MYSQL_KEYRING_PLUGIN         10  /* The Keyring plugin type   */
+#define MYSQL_MAX_PLUGIN_TYPE_NUM    11  /* The number of plugin types   */
 
 /* We use the following strings to define licenses for plugins */
 #define PLUGIN_LICENSE_PROPRIETARY 0
@@ -126,7 +141,7 @@ __MYSQL_DECLARE_PLUGIN(NAME, \
 #define mysql_declare_plugin_end ,{0,0,0,0,0,0,0,0,0,0,0,0,0}}
 
 /**
-  declarations for SHOW STATUS support in plugins
+  Declarations for SHOW STATUS support in plugins
 */
 enum enum_mysql_show_type
 {
@@ -134,17 +149,48 @@ enum enum_mysql_show_type
   SHOW_INT,        ///< shown as _unsigned_ int
   SHOW_LONG,       ///< shown as _unsigned_ long
   SHOW_LONGLONG,   ///< shown as _unsigned_ longlong
+  SHOW_SIGNED_INT,
+  SHOW_SIGNED_LONG,
+  SHOW_SIGNED_LONGLONG,
   SHOW_CHAR, SHOW_CHAR_PTR,
-  SHOW_ARRAY, SHOW_FUNC, SHOW_DOUBLE,
-  SHOW_always_last
+  SHOW_ARRAY, SHOW_FUNC, SHOW_DOUBLE
+#ifdef MYSQL_SERVER
+  /*
+    This include defines server-only values of the enum.
+    Using them in plugins is not supported.
+  */
+  #include "sql_plugin_enum.h"
+#endif
 };
 
-struct st_mysql_show_var {
+/**
+  Status variable scope.
+  Only GLOBAL status variable scope is available in plugins.
+*/
+enum enum_mysql_show_scope
+{
+  SHOW_SCOPE_UNDEF,
+  SHOW_SCOPE_GLOBAL
+#ifdef MYSQL_SERVER
+  /* Server-only values. Not supported in plugins. */
+  ,
+  SHOW_SCOPE_SESSION,
+  SHOW_SCOPE_ALL
+#endif
+};
+
+/**
+  SHOW STATUS Server status variable
+*/
+struct st_mysql_show_var
+{
   const char *name;
   char *value;
   enum enum_mysql_show_type type;
+  enum enum_mysql_show_scope scope;
 };
 
+#define SHOW_VAR_MAX_NAME_LEN 64
 #define SHOW_VAR_FUNC_BUFF_SIZE 1024
 typedef int (*mysql_show_var_func)(MYSQL_THD, struct st_mysql_show_var*, char *);
 
@@ -178,7 +224,9 @@ typedef int (*mysql_show_var_func)(MYSQL_THD, struct st_mysql_show_var*, char *)
 #define PLUGIN_VAR_NOCMDARG     0x1000 /* No argument for cmd line */
 #define PLUGIN_VAR_RQCMDARG     0x0000 /* Argument required for cmd line */
 #define PLUGIN_VAR_OPCMDARG     0x2000 /* Argument optional for cmd line */
+#define PLUGIN_VAR_NODEFAULT    0x4000 /* SET DEFAULT is prohibited */
 #define PLUGIN_VAR_MEMALLOC     0x8000 /* String needs memory allocated */
+#define PLUGIN_VAR_INVISIBLE    0x10000 /* Variable should not be shown */
 
 struct st_mysql_sys_var;
 struct st_mysql_value;
@@ -231,7 +279,8 @@ typedef void (*mysql_var_update_func)(MYSQL_THD thd,
 #define PLUGIN_VAR_MASK \
         (PLUGIN_VAR_READONLY | PLUGIN_VAR_NOSYSVAR | \
          PLUGIN_VAR_NOCMDOPT | PLUGIN_VAR_NOCMDARG | \
-         PLUGIN_VAR_OPCMDARG | PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_MEMALLOC)
+         PLUGIN_VAR_OPCMDARG | PLUGIN_VAR_RQCMDARG | PLUGIN_VAR_MEMALLOC | \
+         PLUGIN_VAR_NODEFAULT | PLUGIN_VAR_INVISIBLE)
 
 #define MYSQL_PLUGIN_VAR_HEADER \
   int flags;                    \
@@ -421,6 +470,10 @@ DECLARE_MYSQL_THDVAR_SIMPLE(name, double) = { \
 #define THDVAR(thd, name) \
   (*(MYSQL_SYSVAR_NAME(name).resolve(thd, MYSQL_SYSVAR_NAME(name).offset)))
 
+#define THDVAR_SET(thd, name, value) \
+  plugin_thdvar_safe_update(thd, MYSQL_SYSVAR(name), \
+                            (char **) &THDVAR(thd, name), \
+                            (const char *) value);
 
 /*
   Plugin description structure.
@@ -446,7 +499,14 @@ struct st_mysql_plugin
 /*************************************************************************
   API for Full-text parser plugin. (MYSQL_FTPARSER_PLUGIN)
 */
-#include "plugin_ftparser.h"
+#define MYSQL_FTPARSER_INTERFACE_VERSION 0x0101
+
+/*************************************************************************
+  API for Query Rewrite plugin. (MYSQL_QUERY_REWRITE_PLUGIN)
+*/
+
+#define MYSQL_REWRITE_PRE_PARSE_INTERFACE_VERSION 0x0010
+#define MYSQL_REWRITE_POST_PARSE_INTERFACE_VERSION 0x0010
 
 /*************************************************************************
   API for Storage Engine plugin. (MYSQL_DAEMON_PLUGIN)
@@ -508,8 +568,8 @@ struct handlerton;
 /*
   API for Replication plugin. (MYSQL_REPLICATION_PLUGIN)
 */
- #define MYSQL_REPLICATION_INTERFACE_VERSION 0x0200
- 
+ #define MYSQL_REPLICATION_INTERFACE_VERSION 0x0400
+
  /**
     Replication plugin descriptor
  */
@@ -553,30 +613,73 @@ int thd_in_lock_tables(const MYSQL_THD thd);
 int thd_tablespace_op(const MYSQL_THD thd);
 long long thd_test_options(const MYSQL_THD thd, long long test_options);
 int thd_sql_command(const MYSQL_THD thd);
-const char *thd_proc_info(MYSQL_THD thd, const char *info);
+const char *set_thd_proc_info(MYSQL_THD thd, const char *info,
+                              const char *calling_func,
+                              const char *calling_file,
+                              const unsigned int calling_line);
 void **thd_ha_data(const MYSQL_THD thd, const struct handlerton *hton);
 void thd_storage_lock_wait(MYSQL_THD thd, long long value);
 int thd_tx_isolation(const MYSQL_THD thd);
 int thd_tx_is_read_only(const MYSQL_THD thd);
-char *thd_security_context(MYSQL_THD thd, char *buffer, unsigned int length,
-                           unsigned int max_query_len);
+MYSQL_THD thd_tx_arbitrate(MYSQL_THD requestor, MYSQL_THD holder);
+int thd_tx_priority(const MYSQL_THD thd);
+int thd_tx_is_dd_trx(const MYSQL_THD thd);
+char *thd_security_context(MYSQL_THD thd, char *buffer, size_t length,
+                           size_t max_query_len);
 /* Increments the row counter, see THD::row_count */
 void thd_inc_row_count(MYSQL_THD thd);
 int thd_allow_batch(MYSQL_THD thd);
 
-void increment_thd_innodb_stats(MYSQL_THD thd,
-                    unsigned long long trx_id,
-                    long io_reads,
-                    long long io_read,
-                    long io_reads_wait_timer,
-                    long lock_que_wait_timer,
-                    long que_wait_timer,
-                    long page_access);
-unsigned long thd_log_slow_verbosity(const MYSQL_THD thd);
-int thd_opt_slow_log();
-#define EXTENDED_SLOWLOG
 
-#define EXTENDED_FOR_USERSTAT
+/**
+  Mark transaction to rollback and mark error as fatal to a
+  sub-statement if in sub statement mode.
+
+  @param thd  user thread connection handle
+  @param all  if all != 0, rollback the main transaction
+*/
+
+void thd_mark_transaction_to_rollback(MYSQL_THD thd, int all);
+
+/** Types of statistics that can be passed to thd_report_innodb_stat */
+enum mysql_trx_stat_type
+{
+  /** Volume of I/O read requests in bytes */
+  MYSQL_TRX_STAT_IO_READ_BYTES,
+  /** Time in microseconds spent waiting for I/O reads to complete */
+  MYSQL_TRX_STAT_IO_READ_WAIT_USECS,
+  /** Time in microseconds spent waiting for row locks */
+  MYSQL_TRX_STAT_LOCK_WAIT_USECS,
+  /** Time in microseconds spent waiting to enter InnoDB */
+  MYSQL_TRX_STAT_INNODB_QUEUE_WAIT_USECS,
+  /** A logical data page accessed */
+  MYSQL_TRX_STAT_ACCESS_PAGE_ID
+};
+
+/**
+  Report various InnoDB statistics for the slow query log extensions
+
+  @param[in]    thd     user thread connection handle
+  @param[in]    trx_id  InnoDB tranaction ID
+  @param[in]    type    type of statistics being reported
+  @param[in]    value   the value of statistics
+*/
+void thd_report_innodb_stat(MYSQL_THD thd, unsigned long long trx_id,
+                            enum mysql_trx_stat_type type,
+                            unsigned long long       value);
+
+unsigned long thd_log_slow_verbosity(const MYSQL_THD thd);
+
+int thd_opt_slow_log();
+
+/**
+  Check whether given connection handle is associated with a background thread.
+
+  @param thd  connection handle
+  @retval non-zero  the connection handle belongs to a background thread
+  @retval 0   the connection handle belongs to a different thread type
+*/
+int thd_is_background_thread(const MYSQL_THD thd);
 
 /**
   Create a temporary file.
@@ -608,6 +711,12 @@ int mysql_tmpfile(const char *prefix);
 */
 int thd_killed(const MYSQL_THD thd);
 
+/**
+  Set the killed status of the current statement.
+
+  @param thd  user thread connection handle
+*/
+void thd_set_kill_status(const MYSQL_THD thd);
 
 /**
   Get binary log position for latest written entry.
@@ -624,13 +733,6 @@ void thd_binlog_pos(const MYSQL_THD thd,
                     const char **file_var,
                     unsigned long long *pos_var);
 
-/**
-  Set the killed status of the current statement.
-
-  @param thd  user thread connection handle
-*/
-void thd_set_kill_status(const MYSQL_THD thd);
-
 /**
   Return the thread id of a user thread
 
@@ -639,6 +741,13 @@ void thd_set_kill_status(const MYSQL_THD thd);
 */
 unsigned long thd_get_thread_id(const MYSQL_THD thd);
 
+/**
+  Return the query id of a thread
+  @param thd user thread
+  @return query id
+*/
+int64_t thd_get_query_id(const MYSQL_THD thd);
+
 /**
   Get the XID for this connection's transaction
 
@@ -651,8 +760,8 @@ void thd_get_xid(const MYSQL_THD thd, MYSQL_XID *xid);
   Invalidate the query cache for a given table.
 
   @param thd         user thread connection handle
-  @param key         databasename\\0tablename\\0
-  @param key_length  length of key in bytes, including the NUL bytes
+  @param key         databasename/tablename in the canonical format.
+  @param key_length  length of key in bytes, including the PATH separator
   @param using_trx   flag: TRUE if using transactions, FALSE otherwise
 */
 void mysql_query_cache_invalidate4(MYSQL_THD thd,
@@ -693,11 +802,17 @@ void thd_set_ha_data(MYSQL_THD thd, const struct handlerton *hton,
 int thd_command(const MYSQL_THD thd);
 long long thd_start_time(const MYSQL_THD thd);
 void thd_kill(unsigned long id);
-#define EXTENDED_FOR_KILLIDLE
+
+/**
+  Check whether ft_query_extra_word_chars server variable is enabled for the
+  current session
+
+  @return ft_query_extra_word_chars value
+*/
+int thd_get_ft_query_extra_word_chars(void);
 
 #ifdef __cplusplus
 }
 #endif
 
-#endif
-
+#endif /* _my_plugin_h */
