@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2013 Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,55 +19,63 @@
 */
 
 #include "mysys_priv.h"
+#include "my_sys.h"
 #include <m_string.h>
 #include <signal.h>
+#include "my_thread_local.h"
 
-pthread_key(struct st_my_thread_var*, THR_KEY_mysys);
-my_bool THR_KEY_mysys_initialized= FALSE;
-mysql_mutex_t THR_LOCK_malloc, THR_LOCK_open,
-              THR_LOCK_lock, THR_LOCK_isam, THR_LOCK_myisam, THR_LOCK_heap,
-              THR_LOCK_net, THR_LOCK_charset, THR_LOCK_threads,
-              THR_LOCK_myisam_mmap;
-
-mysql_cond_t  THR_COND_threads;
-uint            THR_thread_count= 0;
-uint 		my_thread_end_wait_time= 5;
-#if !defined(HAVE_LOCALTIME_R) || !defined(HAVE_GMTIME_R)
-mysql_mutex_t LOCK_localtime_r;
+static my_bool THR_KEY_mysys_initialized= FALSE;
+static my_bool my_thread_global_init_done= FALSE;
+#ifndef DBUG_OFF
+static uint    THR_thread_count= 0;
+static uint    my_thread_end_wait_time= 5;
+static my_thread_id thread_id= 0;
+static thread_local_key_t THR_KEY_mysys;
 #endif
+static thread_local_key_t THR_KEY_myerrno;
+#ifdef _WIN32
+static thread_local_key_t THR_KEY_winerrno;
+#endif
+
+mysql_mutex_t THR_LOCK_malloc, THR_LOCK_open,
+              THR_LOCK_lock, THR_LOCK_myisam, THR_LOCK_heap,
+              THR_LOCK_net, THR_LOCK_charset,
+              THR_LOCK_myisam_mmap;
+#ifndef DBUG_OFF
+mysql_mutex_t THR_LOCK_threads;
+mysql_cond_t  THR_COND_threads;
+#endif
+
 #ifdef PTHREAD_ADAPTIVE_MUTEX_INITIALIZER_NP
-pthread_mutexattr_t my_fast_mutexattr;
+native_mutexattr_t my_fast_mutexattr;
 #endif
 #ifdef PTHREAD_ERRORCHECK_MUTEX_INITIALIZER_NP
-pthread_mutexattr_t my_errorcheck_mutexattr;
+native_mutexattr_t my_errorcheck_mutexattr;
 #endif
-#ifdef _MSC_VER
+#ifdef _WIN32
 static void install_sigabrt_handler();
 #endif
-#ifdef TARGET_OS_LINUX
 
-/*
-  Dummy thread spawned in my_thread_global_init() below to avoid
-  race conditions in NPTL pthread_exit code.
-*/
-
-static pthread_handler_t
-nptl_pthread_exit_hack_handler(void *arg __attribute((unused)))
+#ifndef DBUG_OFF
+struct st_my_thread_var
 {
-  /* Do nothing! */
-  pthread_exit(0);
-  return 0;
+  my_thread_id id;
+  struct _db_code_state_ *dbug;
+};
+
+static struct st_my_thread_var *mysys_thread_var()
+{
+  DBUG_ASSERT(THR_KEY_mysys_initialized);
+  return  (struct st_my_thread_var*)my_get_thread_local(THR_KEY_mysys);
 }
 
-#endif /* TARGET_OS_LINUX */
 
-
-static uint get_thread_lib(void);
-
-static int set_mysys_var(struct st_my_thread_var *mysys_var);
-
-/** True if @c my_thread_global_init() has been called. */
-static my_bool my_thread_global_init_done= 0;
+static int set_mysys_thread_var(struct st_my_thread_var *mysys_var)
+{
+  DBUG_ASSERT(THR_KEY_mysys_initialized);
+  return my_set_thread_local(THR_KEY_mysys, mysys_var);
+}
+#endif
 
 
 /**
@@ -78,19 +86,15 @@ static my_bool my_thread_global_init_done= 0;
   This is safe, since this function() is called before creating new threads,
   so the mutexes are not in use.
 */
-void my_thread_global_reinit(void)
-{
-  struct st_my_thread_var *tmp;
 
+void my_thread_global_reinit()
+{
   DBUG_ASSERT(my_thread_global_init_done);
 
 #ifdef HAVE_PSI_INTERFACE
   my_init_mysys_psi_keys();
 #endif
 
-  mysql_mutex_destroy(&THR_LOCK_isam);
-  mysql_mutex_init(key_THR_LOCK_isam, &THR_LOCK_isam, MY_MUTEX_INIT_SLOW);
-
   mysql_mutex_destroy(&THR_LOCK_heap);
   mysql_mutex_init(key_THR_LOCK_heap, &THR_LOCK_heap, MY_MUTEX_INIT_FAST);
 
@@ -109,40 +113,34 @@ void my_thread_global_reinit(void)
   mysql_mutex_destroy(&THR_LOCK_charset);
   mysql_mutex_init(key_THR_LOCK_charset, &THR_LOCK_charset, MY_MUTEX_INIT_FAST);
 
+#ifndef DBUG_OFF
   mysql_mutex_destroy(&THR_LOCK_threads);
   mysql_mutex_init(key_THR_LOCK_threads, &THR_LOCK_threads, MY_MUTEX_INIT_FAST);
 
   mysql_cond_destroy(&THR_COND_threads);
-  mysql_cond_init(key_THR_COND_threads, &THR_COND_threads, NULL);
-
-  tmp= _my_thread_var();
-  DBUG_ASSERT(tmp);
-
-  mysql_mutex_destroy(&tmp->mutex);
-  mysql_mutex_init(key_my_thread_var_mutex, &tmp->mutex, MY_MUTEX_INIT_FAST);
-
-  mysql_cond_destroy(&tmp->suspend);
-  mysql_cond_init(key_my_thread_var_suspend, &tmp->suspend, NULL);
+  mysql_cond_init(key_THR_COND_threads, &THR_COND_threads);
+#endif
 }
 
-/*
+
+/**
   initialize thread environment
 
-  SYNOPSIS
-    my_thread_global_init()
-
-  RETURN
-    0  ok
-    1  error (Couldn't create THR_KEY_mysys)
+  @retval  FALSE  ok
+  @retval  TRUE   error (Couldn't create THR_KEY_mysys)
 */
 
-my_bool my_thread_global_init(void)
+my_bool my_thread_global_init()
 {
   int pth_ret;
 
   if (my_thread_global_init_done)
-    return 0;
-  my_thread_global_init_done= 1;
+    return FALSE;
+  my_thread_global_init_done= TRUE;
+
+#if defined(SAFE_MUTEX)
+  safe_mutex_global_init();		/* Must be called early */
+#endif
 
 #ifdef PTHREAD_ADAPTIVE_MUTEX_INITIALIZER_NP
   /*
@@ -169,76 +167,57 @@ my_bool my_thread_global_init(void)
 #endif
 
   DBUG_ASSERT(! THR_KEY_mysys_initialized);
-  if ((pth_ret= pthread_key_create(&THR_KEY_mysys, NULL)) != 0)
-  {
-    fprintf(stderr, "Can't initialize threads: error %d\n", pth_ret);
-    return 1;
+#ifndef DBUG_OFF
+  if ((pth_ret= my_create_thread_local_key(&THR_KEY_mysys, NULL)) != 0)
+  { /* purecov: begin inspected */
+    my_message_local(ERROR_LEVEL, "Can't initialize threads: error %d",
+                     pth_ret);
+    /* purecov: end */
+    return TRUE;
   }
-
+#endif
+  if ((pth_ret= my_create_thread_local_key(&THR_KEY_myerrno, NULL)) != 0)
+  { /* purecov: begin inspected */
+    my_message_local(ERROR_LEVEL, "Can't initialize threads: error %d",
+                     pth_ret);
+    /* purecov: end */
+    return TRUE;
+  }
+#ifdef _WIN32
+  if ((pth_ret= my_create_thread_local_key(&THR_KEY_winerrno, NULL)) != 0)
+  { /* purecov: begin inspected */
+    my_message_local(ERROR_LEVEL, "Can't initialize threads: error %d",
+                     pth_ret);
+    /* purecov: end */
+    return TRUE;
+  }
+#endif
   THR_KEY_mysys_initialized= TRUE;
+
   mysql_mutex_init(key_THR_LOCK_malloc, &THR_LOCK_malloc, MY_MUTEX_INIT_FAST);
   mysql_mutex_init(key_THR_LOCK_open, &THR_LOCK_open, MY_MUTEX_INIT_FAST);
   mysql_mutex_init(key_THR_LOCK_charset, &THR_LOCK_charset, MY_MUTEX_INIT_FAST);
-  mysql_mutex_init(key_THR_LOCK_threads, &THR_LOCK_threads, MY_MUTEX_INIT_FAST);
-
-  if (my_thread_init())
-    return 1;
-
-  thd_lib_detected= get_thread_lib();
-
-#ifdef TARGET_OS_LINUX
-  /*
-    BUG#24507: Race conditions inside current NPTL pthread_exit()
-    implementation.
-
-    To avoid a possible segmentation fault during concurrent
-    executions of pthread_exit(), a dummy thread is spawned which
-    initializes internal variables of pthread lib. See bug description
-    for a full explanation.
-
-    TODO: Remove this code when fixed versions of glibc6 are in common
-    use.
-  */
-  if (thd_lib_detected == THD_LIB_NPTL)
-  {
-    pthread_t       dummy_thread;
-    pthread_attr_t  dummy_thread_attr;
-
-    pthread_attr_init(&dummy_thread_attr);
-    pthread_attr_setdetachstate(&dummy_thread_attr, PTHREAD_CREATE_JOINABLE);
-
-    if (pthread_create(&dummy_thread,&dummy_thread_attr,
-                       nptl_pthread_exit_hack_handler, NULL) == 0)
-      (void)pthread_join(dummy_thread, NULL);
-  }
-#endif /* TARGET_OS_LINUX */
-
   mysql_mutex_init(key_THR_LOCK_lock, &THR_LOCK_lock, MY_MUTEX_INIT_FAST);
-  mysql_mutex_init(key_THR_LOCK_isam, &THR_LOCK_isam, MY_MUTEX_INIT_SLOW);
   mysql_mutex_init(key_THR_LOCK_myisam, &THR_LOCK_myisam, MY_MUTEX_INIT_SLOW);
   mysql_mutex_init(key_THR_LOCK_myisam_mmap, &THR_LOCK_myisam_mmap, MY_MUTEX_INIT_FAST);
   mysql_mutex_init(key_THR_LOCK_heap, &THR_LOCK_heap, MY_MUTEX_INIT_FAST);
   mysql_mutex_init(key_THR_LOCK_net, &THR_LOCK_net, MY_MUTEX_INIT_FAST);
-  mysql_cond_init(key_THR_COND_threads, &THR_COND_threads, NULL);
-
-#if !defined(HAVE_LOCALTIME_R) || !defined(HAVE_GMTIME_R)
-  mysql_mutex_init(key_LOCK_localtime_r, &LOCK_localtime_r, MY_MUTEX_INIT_SLOW);
+#ifndef DBUG_OFF
+  mysql_mutex_init(key_THR_LOCK_threads, &THR_LOCK_threads, MY_MUTEX_INIT_FAST);
+  mysql_cond_init(key_THR_COND_threads, &THR_COND_threads);
 #endif
 
-#ifdef _MSC_VER
-  install_sigabrt_handler();
-#endif
-
-  return 0;
+  return FALSE;
 }
 
 
-void my_thread_global_end(void)
+void my_thread_global_end()
 {
+#ifndef DBUG_OFF
   struct timespec abstime;
-  my_bool all_threads_killed= 1;
+  my_bool all_threads_killed= TRUE;
 
-  set_timespec(abstime, my_thread_end_wait_time);
+  set_timespec(&abstime, my_thread_end_wait_time);
   mysql_mutex_lock(&THR_LOCK_threads);
   while (THR_thread_count > 0)
   {
@@ -246,25 +225,33 @@ void my_thread_global_end(void)
                                     &abstime);
     if (error == ETIMEDOUT || error == ETIME)
     {
-#ifdef HAVE_PTHREAD_KILL
+#ifndef _WIN32
       /*
         We shouldn't give an error here, because if we don't have
         pthread_kill(), programs like mysqld can't ensure that all threads
         are killed when we enter here.
       */
       if (THR_thread_count)
-        fprintf(stderr,
-                "Error in my_thread_global_end(): %d threads didn't exit\n",
-                THR_thread_count);
+        /* purecov: begin inspected */
+        my_message_local(ERROR_LEVEL, "Error in my_thread_global_end(): "
+                         "%d threads didn't exit", THR_thread_count);
+        /* purecov: end */
 #endif
-      all_threads_killed= 0;
+      all_threads_killed= FALSE;
       break;
     }
   }
   mysql_mutex_unlock(&THR_LOCK_threads);
+#endif
 
   DBUG_ASSERT(THR_KEY_mysys_initialized);
-  pthread_key_delete(THR_KEY_mysys);
+#ifndef DBUG_OFF
+  my_delete_thread_local_key(THR_KEY_mysys);
+#endif
+  my_delete_thread_local_key(THR_KEY_myerrno);
+#ifdef _WIN32
+  my_delete_thread_local_key(THR_KEY_winerrno);
+#endif
   THR_KEY_mysys_initialized= FALSE;
 #ifdef PTHREAD_ADAPTIVE_MUTEX_INITIALIZER_NP
   pthread_mutexattr_destroy(&my_fast_mutexattr);
@@ -275,147 +262,77 @@ void my_thread_global_end(void)
   mysql_mutex_destroy(&THR_LOCK_malloc);
   mysql_mutex_destroy(&THR_LOCK_open);
   mysql_mutex_destroy(&THR_LOCK_lock);
-  mysql_mutex_destroy(&THR_LOCK_isam);
   mysql_mutex_destroy(&THR_LOCK_myisam);
   mysql_mutex_destroy(&THR_LOCK_myisam_mmap);
   mysql_mutex_destroy(&THR_LOCK_heap);
   mysql_mutex_destroy(&THR_LOCK_net);
   mysql_mutex_destroy(&THR_LOCK_charset);
+#ifndef DBUG_OFF
   if (all_threads_killed)
   {
     mysql_mutex_destroy(&THR_LOCK_threads);
     mysql_cond_destroy(&THR_COND_threads);
   }
-#if !defined(HAVE_LOCALTIME_R) || !defined(HAVE_GMTIME_R)
-  mysql_mutex_destroy(&LOCK_localtime_r);
 #endif
 
-  my_thread_global_init_done= 0;
+  my_thread_global_init_done= FALSE;
 }
 
-static my_thread_id thread_id= 0;
 
-/*
+/**
   Allocate thread specific memory for the thread, used by mysys and dbug
 
-  SYNOPSIS
-    my_thread_init()
+  @note This function may called multiple times for a thread, for example
+  if one uses my_init() followed by mysql_server_init().
 
-  NOTES
-    We can't use mutex_locks here if we are using windows as
-    we may have compiled the program with SAFE_MUTEX, in which
-    case the checking of mutex_locks will not work until
-    the pthread_self thread specific variable is initialized.
-
-   This function may called multiple times for a thread, for example
-   if one uses my_init() followed by mysql_server_init().
-
-  RETURN
-    0  ok
-    1  Fatal error; mysys/dbug functions can't be used
+  @retval FALSE  ok
+  @retval TRUE   Fatal error; mysys/dbug functions can't be used
 */
 
-my_bool my_thread_init(void)
+my_bool my_thread_init()
 {
+#ifndef DBUG_OFF
   struct st_my_thread_var *tmp;
-  my_bool error=0;
+#endif
 
   if (!my_thread_global_init_done)
-    return 1; /* cannot proceed with unintialized library */
+    return TRUE; /* cannot proceed with unintialized library */
 
-#ifdef EXTRA_DEBUG_THREADS
-  fprintf(stderr,"my_thread_init(): thread_id: 0x%lx\n",
-          (ulong) pthread_self());
-#endif  
-
-  if (_my_thread_var())
-  {
-#ifdef EXTRA_DEBUG_THREADS
-    fprintf(stderr,"my_thread_init() called more than once in thread 0x%lx\n",
-            (long) pthread_self());
-#endif    
-    goto end;
-  }
-
-#ifdef _MSC_VER
+#ifdef _WIN32
   install_sigabrt_handler();
 #endif
 
+#ifndef DBUG_OFF
+  if (mysys_thread_var())
+    return FALSE;
+
   if (!(tmp= (struct st_my_thread_var *) calloc(1, sizeof(*tmp))))
-  {
-    error= 1;
-    goto end;
-  }
-  set_mysys_var(tmp);
-  tmp->pthread_self= pthread_self();
-  mysql_mutex_init(key_my_thread_var_mutex, &tmp->mutex, MY_MUTEX_INIT_FAST);
-  mysql_cond_init(key_my_thread_var_suspend, &tmp->suspend, NULL);
-
-#if defined(_GNU_SOURCE) && (_POSIX_C_SOURCE >= 200112L || _XOPEN_SOURCE >= 600)
-  {
-    pthread_attr_t attr;
-    char*          stack_addr;
-    size_t         stack_size;
-
-    if (pthread_attr_init(&attr) ||
-        pthread_getattr_np(tmp->pthread_self, &attr) ||
-        pthread_attr_getstack(&attr, (void**)&stack_addr, &stack_size))
-    {
-      error= 1;
-      goto end;
-    }
-    tmp->stack_ends_here= stack_addr;
-
-    if (pthread_attr_destroy(&attr))
-    {
-      error= 1;
-      goto end;
-    }
-  }
-#else  //defined(_GNU_SOURCE) && (_POSIX_C_SOURCE >= 200112L || _XOPEN_SOURCE >= 600)
-  /* assuming tmp is located within the first page of the stack */
-  tmp->stack_ends_here= (void **)(((long long)&tmp & (~((long long)my_getpagesize() - 1))) +
-                                  ((STACK_DIRECTION == 1) ?
-                                  my_thread_stack_size :
-                                  my_getpagesize() - my_thread_stack_size));
-#endif //defined(_GNU_SOURCE) && (_POSIX_C_SOURCE >= 200112L || _XOPEN_SOURCE >= 600)
+    return TRUE;
 
   mysql_mutex_lock(&THR_LOCK_threads);
   tmp->id= ++thread_id;
   ++THR_thread_count;
   mysql_mutex_unlock(&THR_LOCK_threads);
-  tmp->init= 1;
-#ifndef DBUG_OFF
-  /* Generate unique name for thread */
-  (void) my_thread_name();
+  set_mysys_thread_var(tmp);
 #endif
 
-end:
-  return error;
+  return FALSE;
 }
 
 
-/*
+/**
   Deallocate memory used by the thread for book-keeping
 
-  SYNOPSIS
-    my_thread_end()
-
-  NOTE
-    This may be called multiple times for a thread.
-    This happens for example when one calls 'mysql_server_init()'
-    mysql_server_end() and then ends with a mysql_end().
+  @note This may be called multiple times for a thread.
+  This happens for example when one calls 'mysql_server_init()'
+  mysql_server_end() and then ends with a mysql_end().
 */
 
-void my_thread_end(void)
+void my_thread_end()
 {
-  struct st_my_thread_var *tmp;
-  tmp= _my_thread_var();
-
-#ifdef EXTRA_DEBUG_THREADS
-  fprintf(stderr,"my_thread_end(): tmp: 0x%lx  pthread_self: 0x%lx  thread_id: %ld\n",
-	  (long) tmp, (long) pthread_self(), tmp ? (long) tmp->id : 0L);
-#endif  
+#ifndef DBUG_OFF
+  struct st_my_thread_var *tmp= mysys_thread_var();
+#endif
 
 #ifdef HAVE_PSI_INTERFACE
   /*
@@ -423,28 +340,28 @@ void my_thread_end(void)
     This must be done before trashing st_my_thread_var,
     because the LF_HASH depends on it.
   */
-  if (PSI_server)
-    PSI_server->delete_current_thread();
+  PSI_THREAD_CALL(delete_current_thread)();
 #endif
 
-  if (tmp && tmp->init)
-  {
 #if !defined(DBUG_OFF)
+  if (tmp)
+  {
     /* tmp->dbug is allocated inside DBUG library */
     if (tmp->dbug)
     {
       DBUG_POP();
       free(tmp->dbug);
-      tmp->dbug=0;
+      tmp->dbug= NULL;
     }
-#endif
-#if !defined(__bsdi__) && !defined(__OpenBSD__)
- /* bsdi and openbsd 3.5 dumps core here */
-    mysql_cond_destroy(&tmp->suspend);
-#endif
-    mysql_mutex_destroy(&tmp->mutex);
     free(tmp);
-
+    /*
+     set_mysys_thread_var (NULL) must be called before signaling conditional variable
+     THR_COND_threads. This is variable on which my_thread_global_end is waiting.
+     After it is signalled THR_KEY_mysys_initialized is set to FALSE in my_thread_global_end.
+     There is a debug assertion that THE_KEY_mysys_initialized == TRUE in
+     set_my_sys_thread_var.
+    */
+    set_mysys_thread_var(NULL);
     /*
       Decrement counter for number of running threads. We are using this
       in my_thread_global_end() to wait until all threads have called
@@ -457,56 +374,56 @@ void my_thread_end(void)
       mysql_cond_signal(&THR_COND_threads);
     mysql_mutex_unlock(&THR_LOCK_threads);
   }
-  set_mysys_var(NULL);
+#endif
 }
 
-struct st_my_thread_var *_my_thread_var(void)
-{
-  if (THR_KEY_mysys_initialized)
-    return  my_pthread_getspecific(struct st_my_thread_var*,THR_KEY_mysys);
-  return NULL;
-}
 
-int set_mysys_var(struct st_my_thread_var *mysys_var)
+int my_errno()
 {
   if (THR_KEY_mysys_initialized)
-    return my_pthread_setspecific_ptr(THR_KEY_mysys, mysys_var);
+    return  (int)(intptr)my_get_thread_local(THR_KEY_myerrno);
   return 0;
 }
 
-/****************************************************************************
-  Get name of current thread.
-****************************************************************************/
 
-my_thread_id my_thread_dbug_id()
+void set_my_errno(int my_errno)
 {
-  return my_thread_var->id;
+  if (THR_KEY_mysys_initialized)
+    (void) my_set_thread_local(THR_KEY_myerrno, (void*)(intptr)my_errno);
 }
 
-#ifdef DBUG_OFF
-const char *my_thread_name(void)
+
+#ifdef _WIN32
+int thr_winerr()
 {
-  return "no_name";
+  if (THR_KEY_mysys_initialized)
+    return  (int)(intptr)my_get_thread_local(THR_KEY_winerrno);
+  return 0;
 }
 
-#else
 
-const char *my_thread_name(void)
+void set_thr_winerr(int winerr)
 {
-  char name_buff[100];
-  struct st_my_thread_var *tmp=my_thread_var;
-  if (!tmp->name[0])
-  {
-    my_thread_id id= my_thread_dbug_id();
-    sprintf(name_buff,"T@%lu", (ulong) id);
-    strmake(tmp->name,name_buff,THREAD_NAME_SIZE);
-  }
-  return tmp->name;
+  if (THR_KEY_mysys_initialized)
+    (void) my_set_thread_local(THR_KEY_winerrno, (void*)(intptr)winerr);
+}
+#endif
+
+
+#ifndef DBUG_OFF
+my_thread_id my_thread_var_id()
+{
+  return mysys_thread_var()->id;
 }
 
-/* Return pointer to DBUG for holding current state */
 
-extern void **my_thread_var_dbug()
+void set_my_thread_var_id(my_thread_id id)
+{
+  mysys_thread_var()->id= id;
+}
+
+
+struct _db_code_state_ **my_thread_var_dbug()
 {
   struct st_my_thread_var *tmp;
   /*
@@ -518,27 +435,12 @@ extern void **my_thread_var_dbug()
   */
   if (! THR_KEY_mysys_initialized)
     return NULL;
-  tmp= _my_thread_var();
-  return tmp && tmp->init ? &tmp->dbug : 0;
+  tmp= mysys_thread_var();
+  return tmp ? &tmp->dbug : NULL;
 }
 #endif /* DBUG_OFF */
 
 
-static uint get_thread_lib(void)
-{
-#ifdef _CS_GNU_LIBPTHREAD_VERSION
-  char buff[64];
-    
-  confstr(_CS_GNU_LIBPTHREAD_VERSION, buff, sizeof(buff));
-
-  if (!strncasecmp(buff, "NPTL", 4))
-    return THD_LIB_NPTL;
-  if (!strncasecmp(buff, "linuxthreads", 12))
-    return THD_LIB_LT;
-#endif
-  return THD_LIB_OTHER;
-}
-
 #ifdef _WIN32
 /*
   In Visual Studio 2005 and later, default SIGABRT handler will overwrite
@@ -548,20 +450,16 @@ static uint get_thread_lib(void)
   EXCEPTION_BREAKPOINT and then handle_segfault will do its magic.
 */
 
-#if (_MSC_VER >= 1400)
 static void my_sigabrt_handler(int sig)
 {
   __debugbreak();
 }
-#endif /*_MSC_VER >=1400 */
 
-static void install_sigabrt_handler(void)
+static void install_sigabrt_handler()
 {
-#if (_MSC_VER >=1400)
   /*abort() should not override our exception filter*/
   _set_abort_behavior(0,_CALL_REPORTFAULT);
   signal(SIGABRT,my_sigabrt_handler);
-#endif /* _MSC_VER >=1400 */
 }
 #endif
 
