@@ -1,25 +1,13 @@
 /* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License, version 2.0,
-   as published by the Free Software Foundation.
-
-   This program is also distributed with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
-   as designated in a particular file or component or in included license
-   documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
-
-   Without limiting anything contained in the foregoing, this file,
-   which is part of C Driver for MySQL (Connector/C), is also subject to the
-   Universal FOSS Exception, version 1.0, a copy of which can be found at
-   http://oss.oracle.com/licenses/universal-foss-exception.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License, version 2.0, for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -55,14 +43,18 @@
 #include "m_ctype.h"
 #include <my_dir.h>
 #include <my_aes.h>
-#ifdef __WIN__
+#include "mysql/psi/mysql_file.h"
+#ifdef _WIN32
 #include <winbase.h>
 #endif
 
+#include "prealloced_array.h"
+
 C_MODE_START
 #ifdef HAVE_PSI_INTERFACE
 extern PSI_file_key key_file_cnf;
 #endif
+PSI_memory_key key_memory_defaults;
 C_MODE_END
 
 /**
@@ -126,7 +118,7 @@ static my_bool defaults_already_read= FALSE;
 #define DEFAULT_DIRS_SIZE (MAX_DEFAULT_DIRS + 1)  /* Terminate with NULL */
 static const char **default_directories = NULL;
 
-#ifdef __WIN__
+#ifdef _WIN32
 static const char *f_extensions[]= { ".ini", ".cnf", 0 };
 #define NEWLINE "\r\n"
 #else
@@ -134,8 +126,10 @@ static const char *f_extensions[]= { ".cnf", 0 };
 #define NEWLINE "\n"
 #endif
 
+extern "C" {
 static int handle_default_option(void *in_ctx, const char *group_name,
                                  const char *option);
+}
 
 /*
    This structure defines the context that we pass to callback
@@ -144,10 +138,11 @@ static int handle_default_option(void *in_ctx, const char *group_name,
    was called from load_defaults.
 */
 
+typedef Prealloced_array<char*, 100> My_args;
 struct handle_option_ctx
 {
    MEM_ROOT *alloc;
-   DYNAMIC_ARRAY *args;
+   My_args *m_args;
    TYPELIB *group;
 };
 
@@ -404,8 +399,9 @@ int my_search_option_files(const char *conf_file, int *argc, char ***argv,
       goto err;
     if (error > 0)
     {
-      fprintf(stderr, "Could not open required defaults file: %s\n",
-              my_defaults_file);
+      my_message_local(ERROR_LEVEL,
+                       "Could not open required defaults file: %s",
+                       my_defaults_file);
       goto err;
     }
   }
@@ -427,8 +423,9 @@ int my_search_option_files(const char *conf_file, int *argc, char ***argv,
           goto err;				/* Fatal error */
         if (error > 0)
         {
-          fprintf(stderr, "Could not open required defaults file: %s\n",
-                  my_defaults_extra_file);
+          my_message_local(ERROR_LEVEL,
+                           "Could not open required defaults file: %s",
+                           my_defaults_extra_file);
           goto err;
         }
       }
@@ -438,7 +435,8 @@ int my_search_option_files(const char *conf_file, int *argc, char ***argv,
   DBUG_RETURN(0);
 
 err:
-  fprintf(stderr,"Fatal error in defaults handling. Program aborted\n");
+  my_message_local(ERROR_LEVEL,
+                   "Fatal error in defaults handling. Program aborted!");
   DBUG_RETURN(1);
 }
 
@@ -479,9 +477,9 @@ static int handle_default_option(void *in_ctx, const char *group_name,
   {
     if (!(tmp= (char *) alloc_root(ctx->alloc, strlen(option) + 1)))
       return 1;
-    if (insert_dynamic(ctx->args, &tmp))
+    if (ctx->m_args->push_back(tmp))
       return 1;
-    strmov(tmp, option);
+    my_stpcpy(tmp, option);
   }
 
   return 0;
@@ -633,7 +631,7 @@ my_bool my_defaults_read_login_file= TRUE;
 int my_load_defaults(const char *conf_file, const char **groups,
                   int *argc, char ***argv, const char ***default_directories)
 {
-  DYNAMIC_ARRAY args;
+  My_args my_args(key_memory_defaults);
   TYPELIB group;
   my_bool found_print_defaults= 0;
   uint args_used= 0;
@@ -647,7 +645,7 @@ int my_load_defaults(const char *conf_file, const char **groups,
   uint args_sep= my_getopt_use_args_separator ? 1 : 0;
   DBUG_ENTER("load_defaults");
 
-  init_alloc_root(&alloc,512,0);
+  init_alloc_root(key_memory_defaults, &alloc,512,0);
   if ((dirs= init_default_directories(&alloc)) == NULL)
     goto err;
   /*
@@ -664,18 +662,14 @@ int my_load_defaults(const char *conf_file, const char **groups,
   for (; *groups ; groups++)
     group.count++;
 
-  if (my_init_dynamic_array(&args, sizeof(char*),*argc, 32))
-    goto err;
-
   ctx.alloc= &alloc;
-  ctx.args= &args;
+  ctx.m_args= &my_args;
   ctx.group= &group;
 
   if ((error= my_search_option_files(conf_file, argc, argv,
                                      &args_used, handle_default_option,
                                      (void *) &ctx, dirs, false, found_no_defaults)))
   {
-    delete_dynamic(&args);
     free_root(&alloc,MYF(0));
     DBUG_RETURN(error);
   }
@@ -688,27 +682,24 @@ int my_load_defaults(const char *conf_file, const char **groups,
                                      handle_default_option, (void *) &ctx,
                                      dirs, true, found_no_defaults)))
     {
-      delete_dynamic(&args);
       free_root(&alloc, MYF(0));
       DBUG_RETURN(error);
     }
   }
-
   /*
     Here error contains <> 0 only if we have a fully specified conf_file
     or a forced default file
   */
-  if (!(ptr=(char*) alloc_root(&alloc,sizeof(alloc)+
-			       (args.elements + *argc + 1 + args_sep) *sizeof(char*))))
-  {
-    delete_dynamic(&args);
+  if (!(ptr=(char*)
+        alloc_root(&alloc,sizeof(alloc)+
+                   (my_args.size() + *argc + 1 + args_sep) *sizeof(char*))))
     goto err;
-  }
   res= (char**) (ptr+sizeof(alloc));
 
   /* copy name + found arguments + command line arguments to new array */
   res[0]= argv[0][0];  /* Name MUST be set, even by embedded library */
-  memcpy((uchar*) (res+1), args.buffer, args.elements*sizeof(char*));
+  if (!my_args.empty())
+    memcpy((res+1), &my_args[0], my_args.size() * sizeof(char*));
   /* Skip --defaults-xxx options */
   (*argc)-= args_used;
   (*argv)+= args_used;
@@ -727,18 +718,18 @@ int my_load_defaults(const char *conf_file, const char **groups,
   {
     /* set arguments separator for arguments from config file and
        command line */
-    set_args_separator(&res[args.elements+1]);
+    set_args_separator(&res[my_args.size() + 1]);
   }
 
   if (*argc)
-    memcpy((uchar*) (res+1+args.elements+args_sep), (char*) ((*argv)+1),
+    memcpy((uchar*) (res + 1 + my_args.size() + args_sep),
+           (char*) ((*argv)+1),
 	   (*argc-1)*sizeof(char*));
-  res[args.elements+ *argc+args_sep]=0;                /* last null */
+  res[my_args.size() + *argc + args_sep]= 0;  /* last null */
 
-  (*argc)+=args.elements+args_sep;
-  *argv= (char**) res;
+  (*argc)+= my_args.size() + args_sep;
+  *argv= res;
   *(MEM_ROOT*) ptr= alloc;			/* Save alloc root for free */
-  delete_dynamic(&args);
 
   if (default_directories)
     *default_directories= dirs;
@@ -753,7 +744,12 @@ int my_load_defaults(const char *conf_file, const char **groups,
 	   **argv);
     for (i=1 ; i < *argc ; i++)
       if (!my_getopt_is_args_separator((*argv)[i])) /* skip arguments separator */
-        printf("%s ", (*argv)[i]);
+      {
+        if(strncmp((*argv)[i], "--password", 10) == 0)
+          printf("%s ", "--password=*****");
+        else
+          printf("%s ", (*argv)[i]);
+      }
     puts("");
     exit(0);
   }
@@ -761,7 +757,8 @@ int my_load_defaults(const char *conf_file, const char **groups,
   DBUG_RETURN(0);
 
  err:
-  fprintf(stderr,"Fatal error in defaults handling. Program aborted\n");
+  my_message_local(ERROR_LEVEL,
+                   "Fatal error in defaults handling. Program aborted!");
   exit(1);
   DBUG_RETURN(0); /* Keep compiler happy */
 }
@@ -839,9 +836,9 @@ static char *get_argument(const char *keyword, size_t kwlen,
   /* Print error msg if there is nothing after !include* directive */
   if (end <= ptr)
   {
-    fprintf(stderr,
-	    "error: Wrong '!%s' directive in config file: %s at line %d\n",
-	    keyword, name, line);
+    my_message_local(ERROR_LEVEL,
+                     "Wrong '!%s' directive in config file %s at line %d!",
+                     keyword, name, line);
     return 0;
   }
   return ptr;
@@ -902,7 +899,7 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
   }
   else
   {
-    strmov(name,config_file);
+    my_stpcpy(name,config_file);
   }
   fn_format(name,name,"","",4);
 
@@ -941,10 +938,10 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
              end--)
         {}
         end[0]= 0;
-        fprintf(stderr,
-                "Warning: skipping '%s' directive as maximum include"
-                "recursion level was reached in file %s at line %d\n",
-                ptr, name, line);
+        my_message_local(WARNING_LEVEL,
+                         "skipping '%s' directive as maximum include"
+                         "recursion level was reached in file %s at line %d!",
+                         ptr, name, line);
         continue;
       }
 
@@ -964,7 +961,7 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
         if (!(search_dir= my_dir(ptr, MYF(MY_WME))))
           goto err;
 
-        for (i= 0; i < (uint) search_dir->number_off_files; i++)
+        for (i= 0; i < search_dir->number_off_files; i++)
         {
           search_file= search_dir->dir_entry + i;
           ext= fn_ext(search_file->name);
@@ -1006,12 +1003,12 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
     if (*ptr == '[')				/* Group name */
     {
       found_group=1;
-      if (!(end=(char *) strchr(++ptr,']')))
+      if (!(end= strchr(++ptr,']')))
       {
-	fprintf(stderr,
-		"error: Wrong group definition in config file: %s at line %d\n",
-		name,line);
-	goto err;
+        my_message_local(ERROR_LEVEL,
+                         "Wrong group definition in config file %s at line %d!",
+                         name,line);
+        goto err;
       }
       /* Remove end space */
       for ( ; my_isspace(&my_charset_latin1, end[-1]); end --)
@@ -1028,13 +1025,13 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
     }
     if (!found_group)
     {
-      fprintf(stderr,
-	      "error: Found option without preceding group in config file: %s at line: %d\n",
-	      name,line);
+      my_message_local(ERROR_LEVEL,
+                       "Found option without preceding group in config file"
+                       " %s at line %d!", name,line);
       goto err;
     }
-    
-   
+
+
     end= remove_end_comment(ptr);
     if ((value= strchr(ptr, '=')))
       end= value;				/* Option without argument */
@@ -1043,7 +1040,7 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
 
     if (!value)
     {
-      strmake(strmov(option,"--"),ptr, (size_t) (end-ptr));
+      strmake(my_stpcpy(option,"--"),ptr, (size_t) (end-ptr));
       if (opt_handler(handler_ctx, curr_gr, option))
         goto err;
     }
@@ -1073,7 +1070,7 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
 	value++;
 	value_end--;
       }
-      ptr=strnmov(strmov(option,"--"),ptr,(size_t) (end-ptr));
+      ptr=my_stpnmov(my_stpcpy(option,"--"),ptr,(size_t) (end-ptr));
       *ptr++= '=';
 
       for ( ; value != value_end; value++)
@@ -1231,7 +1228,7 @@ void my_print_default_files(const char *conf_file)
   {
     const char **dirs;
     MEM_ROOT alloc;
-    init_alloc_root(&alloc,512,0);
+    init_alloc_root(key_memory_defaults, &alloc, 512, 0);
 
     if ((dirs= init_default_directories(&alloc)) == NULL)
     {
@@ -1321,7 +1318,7 @@ static int add_directory(MEM_ROOT *alloc, const char *dir, const char **dirs)
 }
 
 
-#ifdef __WIN__
+#ifdef _WIN32
 /*
   This wrapper for GetSystemWindowsDirectory() will dynamically bind to the
   function if it is available, emulate it on NT4 Terminal Server by stripping
@@ -1386,7 +1383,7 @@ static const char *my_get_module_parent(char *buf, size_t size)
 
   return buf;
 }
-#endif /* __WIN__ */
+#endif /* _WIN32 */
 
 
 static const char **init_default_directories(MEM_ROOT *alloc)
@@ -1400,7 +1397,7 @@ static const char **init_default_directories(MEM_ROOT *alloc)
     return NULL;
   memset(dirs, 0, DEFAULT_DIRS_SIZE * sizeof(char *));
 
-#ifdef __WIN__
+#ifdef _WIN32
 
   {
     char fname_buffer[FN_REFLEN];
@@ -1434,7 +1431,7 @@ static const char **init_default_directories(MEM_ROOT *alloc)
   /* Placeholder for --defaults-extra-file=<path> */
   errors += add_directory(alloc, "", dirs);
 
-#if !defined(__WIN__)
+#if !defined(_WIN32)
   errors += add_directory(alloc, "~/", dirs);
 #endif
 
@@ -1491,7 +1488,7 @@ int my_default_get_login_file(char *file_name, size_t file_name_size)
 */
 int check_file_permissions(const char *file_name, my_bool is_login_file)
 {
-#if !defined(__WIN__)
+#if !defined(_WIN32)
   MY_STAT stat_info;
 
   if (!my_stat(file_name,&stat_info,MYF(0)))
@@ -1503,8 +1500,8 @@ int check_file_permissions(const char *file_name, my_bool is_login_file)
   if (is_login_file && (stat_info.st_mode & (S_IXUSR | S_IRWXG | S_IRWXO))
       && (stat_info.st_mode & S_IFMT) == S_IFREG)
   {
-    fprintf(stderr, "Warning: %s should be readable/writable only by "
-            "current user.\n", file_name);
+    my_message_local(WARNING_LEVEL, "%s should be readable/writable only by "
+            "current user.", file_name);
     return 0;
   }
   /*
@@ -1516,8 +1513,8 @@ int check_file_permissions(const char *file_name, my_bool is_login_file)
            (stat_info.st_mode & S_IFMT) == S_IFREG)
 
   {
-    fprintf(stderr, "Warning: World-writable config file '%s' is ignored\n",
-            file_name);
+    my_message_local(WARNING_LEVEL,
+                     "World-writable config file '%s' is ignored.", file_name);
     return 0;
   }
 #endif
