@@ -70,6 +70,7 @@ static char *audit_log_exclude_databases= NULL;
 static char *audit_log_include_databases= NULL;
 static char *audit_log_exclude_commands= NULL;
 static char *audit_log_include_commands= NULL;
+int64 audit_log_buffer_size_overflow = 0;
 
 PSI_memory_key key_memory_audit_log_logger_handle;
 PSI_memory_key key_memory_audit_log_handler;
@@ -176,37 +177,48 @@ typedef struct
 
 static
 void escape_buf(const char *in, size_t *inlen, char *out, size_t *outlen,
-                const escape_rule_t *escape_rules)
+                const escape_rule_t *control_escape_rules,
+                const escape_rule_t *other_escape_rules)
 {
   char* outstart = out;
   const char* base = in;
   char* outend = out + *outlen;
   const char* inend;
-  const escape_rule_t *rule;
-  my_bool replaced;
+  const escape_rule_t *replace_rule = NULL;
 
   inend = in + (*inlen);
 
   while ((in < inend) && (out < outend))
   {
-    replaced= FALSE;
-    for (rule= escape_rules; rule->character; rule++)
+    replace_rule = NULL;
+    if ((unsigned char)(*in) < 32) {
+      if (control_escape_rules[(unsigned int)*in].character) {
+        replace_rule = &control_escape_rules[(unsigned int)*in];
+      }
+    } else
     {
-      if (*in == rule->character)
+      const escape_rule_t *rule = NULL;
+      for (rule= other_escape_rules; rule->character; rule++)
       {
-        if ((outend - out) < (int) rule->length)
-          goto end_of_buffer;
-        memcpy(out, rule->replacement, rule->length);
-        out += rule->length;
-        replaced= TRUE;
-        break;
+        if (*in == rule->character)
+        {
+          replace_rule = rule;
+          break;
+        }
       }
     }
-    if (!replaced)
+    if (replace_rule)
+    {
+          if ((outend - out) < (ptrdiff_t) replace_rule->length)
+            break;
+          memcpy(out, replace_rule->replacement, replace_rule->length);
+          out += replace_rule->length;
+    } else
+    {
       *out++ = *in;
+    }
     ++in;
   }
-end_of_buffer:
   *outlen = out - outstart;
   *inlen = in - base;
 }
@@ -214,45 +226,153 @@ end_of_buffer:
 static
 void xml_escape(const char *in, size_t *inlen, char *out, size_t *outlen)
 {
-  const escape_rule_t rules[]=
+  // Most control sequences aren't supported before XML 1.1, and most
+  // tools only support 1.0. Our output is 1.0. Escaping them wouldn't make
+  // the output more valid.
+  static const escape_rule_t control_rules[]=
+  {
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { '\t', 5, "&#9;" },
+    { '\n', 6, "&#10;" },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { '\r', 6, "&#13;" },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+  };
+  static const escape_rule_t other_rules[]=
   {
     { '<',  4, "&lt;" },
     { '>',  4, "&gt;" },
     { '&',  5, "&amp;" },
-    { '\r', 5, "&#13;" },
-    { '\n', 5, "&#10;" },
     { '"',  6, "&quot;" },
     { 0,  0, NULL }
   };
 
-  escape_buf(in, inlen, out, outlen, rules);
+  escape_buf(in, inlen, out, outlen, control_rules, other_rules);
 }
 
 static
 void json_escape(const char *in, size_t *inlen, char *out, size_t *outlen)
 {
-  const escape_rule_t rules[]=
+  static const escape_rule_t control_rules[]=
+  {
+    { 0,  6, "\\u0000" },
+    { 1,  6, "\\u0001" },
+    { 2,  6, "\\u0002" },
+    { 3,  6, "\\u0003" },
+    { 4,  6, "\\u0004" },
+    { 5,  6, "\\u0005" },
+    { 6,  6, "\\u0006" },
+    { 7,  6, "\\u0007" },
+    { '\b',  2, "\\b" },
+    { '\t',  2, "\\t" },
+    { '\n',  2, "\\n" },
+    { 11, 6, "\\u000B" },
+    { '\f',  2, "\\f" },
+    { '\r',  2, "\\r" },
+    { 14, 6, "\\u000E" },
+    { 15, 6, "\\u000F" },
+    { 16, 6, "\\u0010" },
+    { 17, 6, "\\u0011" },
+    { 18, 6, "\\u0012" },
+    { 19, 6, "\\u0013" },
+    { 20, 6, "\\u0014" },
+    { 21, 6, "\\u0015" },
+    { 22, 6, "\\u0016" },
+    { 23, 6, "\\u0017" },
+    { 24, 6, "\\u0018" },
+    { 25, 6, "\\u0019" },
+    { 26, 6, "\\u001A" },
+    { 27, 6, "\\u001B" },
+    { 28, 6, "\\u001C" },
+    { 29, 6, "\\u001D" },
+    { 30, 6, "\\u001E" },
+    { 31, 6, "\\u001F" },
+  };
+
+  static const escape_rule_t other_rules[]=
   {
     { '\\', 2, "\\\\" },
     { '"',  2, "\\\"" },
-    { '\r',  2, "\\r" },
-    { '\n',  2, "\\n" },
+    { '/',  2, "\\/" },
     { 0,  0, NULL }
   };
 
-  escape_buf(in, inlen, out, outlen, rules);
+  escape_buf(in, inlen, out, outlen, control_rules, other_rules);
 }
 
 static
 void csv_escape(const char *in, size_t *inlen, char *out, size_t *outlen)
 {
-  const escape_rule_t rules[]=
+  // We do not have any standard control escape rules for CSVs
+  static const escape_rule_t control_rules[]=
+  {
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+  };
+
+  static const escape_rule_t other_rules[]=
   {
     { '"',  2, "\"\"" },
     { 0,  0, NULL }
   };
 
-  escape_buf(in, inlen, out, outlen, rules);
+  escape_buf(in, inlen, out, outlen, control_rules, other_rules);
 }
 
 static const escape_buf_func_t format_escape_func[]=
@@ -528,7 +648,8 @@ char *audit_log_general_record(char *buf, size_t buflen,
     endptr= endbuf;
     query= escape_string(event->general_query.str, event->general_query.length,
                          endptr, endbuf - endptr, &endptr, &full_outlen);
-    full_outlen+= full_outlen * my_charset_utf8mb4_general_ci.mbmaxlen;
+    full_outlen*= my_charset_utf8mb4_general_ci.mbmaxlen;
+    full_outlen+= query_length * my_charset_utf8mb4_general_ci.mbmaxlen;
   }
 
   user= escape_string(event->general_user.str, event->general_user.length,
@@ -848,15 +969,91 @@ int audit_log_plugin_init(MYSQL_PLUGIN plugin_info)
   mysql_memory_register(AUDIT_LOG_PSI_CATEGORY, all_audit_log_memory, count);
   logger_init_mutexes();
 
+  audit_log_filter_init();
+
+  if (audit_log_exclude_accounts != NULL && audit_log_include_accounts != NULL)
+  {
+    my_plugin_log_message(&plugin_ptr, MY_ERROR_LEVEL,
+                          "Both 'audit_log_exclude_accounts' and "
+                          "'audit_log_include_accounts' are not NULL\n");
+    goto validation_error;
+  }
+
+  if (audit_log_exclude_commands != NULL && audit_log_include_commands != NULL)
+  {
+    my_plugin_log_message(&plugin_ptr, MY_ERROR_LEVEL,
+                          "Both 'audit_log_exclude_commands' and "
+                          "'audit_log_include_commands' are not NULL\n");
+    goto validation_error;
+  }
+
+  if (audit_log_exclude_databases != NULL
+      && audit_log_include_databases != NULL)
+  {
+    my_plugin_log_message(&plugin_ptr, MY_ERROR_LEVEL,
+                          "Both 'audit_log_exclude_databases' and "
+                          "'audit_log_include_databases' are not NULL\n");
+    goto validation_error;
+  }
+
+  if (audit_log_exclude_accounts != NULL)
+  {
+    audit_log_exclude_accounts= my_strdup(PSI_NOT_INSTRUMENTED,
+                                          audit_log_exclude_accounts,
+                                          MYF(MY_FAE));
+    audit_log_set_exclude_accounts(audit_log_exclude_accounts);
+  }
+  if (audit_log_include_accounts != NULL)
+  {
+    audit_log_include_accounts= my_strdup(PSI_NOT_INSTRUMENTED,
+                                          audit_log_include_accounts,
+                                          MYF(MY_FAE));
+    audit_log_set_include_accounts(audit_log_include_accounts);
+  }
+  if (audit_log_exclude_commands != NULL)
+  {
+    audit_log_exclude_commands= my_strdup(PSI_NOT_INSTRUMENTED,
+                                          audit_log_exclude_commands,
+                                          MYF(MY_FAE));
+    audit_log_set_exclude_commands(audit_log_exclude_commands);
+  }
+  if (audit_log_include_commands != NULL)
+  {
+    audit_log_include_commands= my_strdup(PSI_NOT_INSTRUMENTED,
+                                          audit_log_include_commands,
+                                          MYF(MY_FAE));
+    audit_log_set_include_commands(audit_log_include_commands);
+  }
+  if (audit_log_exclude_databases != NULL)
+  {
+    audit_log_exclude_databases= my_strdup(PSI_NOT_INSTRUMENTED,
+                                          audit_log_exclude_databases,
+                                          MYF(MY_FAE));
+    audit_log_set_exclude_databases(audit_log_exclude_databases);
+  }
+  if (audit_log_include_databases != NULL)
+  {
+    audit_log_include_databases= my_strdup(PSI_NOT_INSTRUMENTED,
+                                          audit_log_include_databases,
+                                          MYF(MY_FAE));
+    audit_log_set_include_databases(audit_log_include_databases);
+  }
+
   if (init_new_log_file())
     return(1);
 
   if (audit_log_audit_record(buf, sizeof(buf), "Audit", time(NULL), &len))
     audit_log_write(buf, len);
 
-  audit_log_filter_init();
+  return 0;
 
-  return(0);
+validation_error:
+
+  audit_log_exclude_accounts= audit_log_include_accounts= NULL;
+  audit_log_exclude_commands= audit_log_include_commands= NULL;
+  audit_log_exclude_databases= audit_log_include_databases= NULL;
+
+  return 1;
 }
 
 
@@ -1174,7 +1371,7 @@ int audit_log_notify(MYSQL_THD thd MY_ATTRIBUTE((unused)),
                                         &len);
       if (len > buflen)
       {
-        buflen= len * 4;
+        buflen= len + 1024;
         log_rec= audit_log_general_record(get_record_buffer(thd, buflen),
                                           buflen,
                                           event_general->general_command.str,
@@ -1182,6 +1379,7 @@ int audit_log_notify(MYSQL_THD thd MY_ATTRIBUTE((unused)),
                                           event_general->general_error_code,
                                           event_general, local->db,
                                           &len);
+        DBUG_ASSERT(log_rec);
       }
       if (log_rec)
         audit_log_write(log_rec, len);
@@ -1868,6 +2066,9 @@ static struct st_mysql_audit audit_log_descriptor=
 
 static struct st_mysql_show_var audit_log_status_variables[]=
 {
+  {"Audit_log_buffer_size_overflow",
+    (char*) &audit_log_buffer_size_overflow,
+    SHOW_LONG, SHOW_SCOPE_GLOBAL},
   {NullS, NullS, SHOW_LONG, SHOW_SCOPE_GLOBAL}
 };
 
