@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -574,8 +574,8 @@ void xcom_cleanup_ssl()
     return;
 
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
-  ERR_remove_state(0);
-#endif
+  ERR_remove_thread_state(0);
+#endif /* OPENSSL_VERSION_NUMBER < 0x10100000L */
 }
 
 void xcom_destroy_ssl()
@@ -686,10 +686,10 @@ int ssl_verify_server_cert(SSL *ssl, const char* server_hostname)
   }
 
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
-  cn= (const char *) ASN1_STRING_data(cn_asn1);
-#else
-  cn= (const char *) ASN1_STRING_get0_data(cn_asn1);
-#endif
+  cn= (char *) ASN1_STRING_data(cn_asn1);
+#else /* OPENSSL_VERSION_NUMBER < 0x10100000L */
+  cn= (char *) ASN1_STRING_get0_data(cn_asn1);
+#endif /* OPENSSL_VERSION_NUMBER < 0x10100000L */
 
   /* There should not be any NULL embedded in the CN */
   if ((size_t)ASN1_STRING_length(cn_asn1) != strlen(cn))
