@@ -100,7 +100,7 @@ extern "C" void handle_fatal_signal(int sig)
     "As this is a crash and something is definitely wrong, the information\n"
     "collection process might fail.\n"
     "Please help us make Percona Server better by reporting any\n"
-    "bugs at http://bugs.percona.com/\n\n");
+    "bugs at https://bugs.percona.com/\n\n");
 
   my_safe_printf_stderr("key_buffer_size=%lu\n",
                         (ulong) dflt_key_cache->key_cache_mem_size);
