@@ -2173,7 +2173,7 @@ void sp_head::add_used_tables_to_table_list(THD *thd,
 bool sp_head::check_show_access(THD *thd, bool *full_access)
 {
   TABLE_LIST tables;
-  memset(static_cast<void*>(&tables), 0, sizeof(tables));
+
   tables.db= (char*) "mysql";
   tables.table_name= tables.alias= (char*) "proc";
 
@@ -2236,6 +2236,8 @@ void sp_parser_data::start_parsing_sp_body(THD *thd, sp_head *sp)
   m_saved_free_list= thd->free_list;
 
   thd->mem_root= sp->get_persistent_mem_root();
+  set_memroot_max_capacity(thd->mem_root, m_saved_memroot->max_capacity);
+  set_memroot_error_reporting(thd->mem_root, m_saved_memroot->error_for_capacity_exceeded);
   thd->free_list= NULL;
 }
 
