@@ -50,7 +50,9 @@ PageBulk::init()
 	mtr = static_cast<mtr_t*>(
 		mem_heap_alloc(m_heap, sizeof(mtr_t)));
 	mtr_start(mtr);
-	mtr_x_lock(dict_index_get_lock(m_index), mtr);
+	if (m_index->is_committed()) {
+		mtr_x_lock(dict_index_get_lock(m_index), mtr);
+	}
 	mtr_set_log_mode(mtr, MTR_LOG_NO_REDO);
 	mtr_set_flush_observer(mtr, m_flush_observer);
 
@@ -333,6 +335,10 @@ PageBulk::compress()
 {
 	ut_ad(m_page_zip != NULL);
 
+	DBUG_EXECUTE_IF("innodb_bulk_load_compress_sleep",
+		os_thread_sleep(1000000);
+	);
+
 	return(page_zip_compress(m_page_zip, m_page, m_index,
 				 page_zip_level, NULL, m_mtr));
 }
@@ -604,7 +610,9 @@ PageBulk::latch()
 	ibool	ret;
 
 	mtr_start(m_mtr);
-	mtr_x_lock(dict_index_get_lock(m_index), m_mtr);
+	if (m_index->is_committed()) {
+		mtr_x_lock(dict_index_get_lock(m_index), m_mtr);
+	}
 	mtr_set_log_mode(m_mtr, MTR_LOG_NO_REDO);
 	mtr_set_flush_observer(m_mtr, m_flush_observer);
 
@@ -791,6 +799,8 @@ BtrBulk::insert(
 			return(err);
 		}
 
+		DEBUG_SYNC_C("bulk_load_insert");
+
 		m_page_bulks->push_back(new_page_bulk);
 		ut_ad(level + 1 == m_page_bulks->size());
 		m_root_level = level;
