@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2018, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1996, 2019, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2012, Facebook Inc.
 
 This program is free software; you can redistribute it and/or modify it under
@@ -171,6 +171,8 @@ dict_mem_table_create(
 
 	table->is_corrupt = false;
 
+	table->file_unreadable = false;
+
 #endif /* !UNIV_HOTBACKUP */
 
 	if (DICT_TF_HAS_SHARED_SPACE(table->flags)) {
@@ -230,6 +232,7 @@ dict_mem_table_free(
 	}
 
 	mem_heap_free(table->heap);
+        table = NULL;
 }
 
 /****************************************************************//**
