@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -876,14 +876,12 @@ int ha_myisam::write_row(uchar *buf)
     If we have an auto_increment column and we are writing a changed row
     or a new row, then update the auto_increment value in the record.
   */
-  if (table->next_number_field && buf == table->record[0])
+  if (table && table->next_number_field && buf == table->record[0])
   {
     if ((error= update_auto_increment()))
       return error;
   }
   error=mi_write(file,buf);
-  if (!error)
-    rows_changed++;
   return error;
 }
 
@@ -1227,10 +1225,14 @@ int ha_myisam::repair(THD *thd, MI_CHECK &param, bool do_optimize)
     if (file->s->base.auto_key)
       update_auto_increment_key(&param, file, 1);
     if (optimize_done)
+    {
+      mysql_mutex_lock(&share->intern_lock);
       error = update_state_info(&param, file,
 				UPDATE_TIME | UPDATE_OPEN_COUNT |
 				(local_testflag &
 				 T_STATISTICS ? UPDATE_STAT : 0));
+      mysql_mutex_unlock(&share->intern_lock);
+    }
     info(HA_STATUS_NO_LOCK | HA_STATUS_TIME | HA_STATUS_VARIABLE |
 	 HA_STATUS_CONST);
     if (rows != file->state->records && ! (param.testflag & T_VERY_SILENT))
@@ -1660,8 +1662,6 @@ int ha_myisam::update_row(const uchar *old_data, uchar *new_data)
   int error;
   ha_statistic_increment(&SSV::ha_update_count);
   error=mi_update(file,old_data,new_data);
-  if (!error)
-    rows_changed++;
   return error;
 }
 
@@ -1670,8 +1670,6 @@ int ha_myisam::delete_row(const uchar *buf)
   int error;
   ha_statistic_increment(&SSV::ha_delete_count);
   error=mi_delete(file,buf);
-  if (!error)
-    rows_changed++;
   return error;
 }
 
@@ -1724,14 +1722,6 @@ int ha_myisam::index_read_map(uchar *buf, const uchar *key,
   ha_statistic_increment(&SSV::ha_read_key_count);
   int error=mi_rkey(file, buf, active_index, key, keypart_map, find_flag);
   table->status=error ? STATUS_NOT_FOUND: 0;
-  if (!error)
-  {
-    rows_read++;
-
-    int inx = (active_index == MAX_KEY) ? file->lastinx : active_index;
-    if (inx >= 0 && inx < (int)MAX_KEY)
-      index_rows_read[inx]++;
-  }
   MYSQL_INDEX_READ_ROW_DONE(error);
   return error;
 }
@@ -1746,14 +1736,6 @@ int ha_myisam::index_read_idx_map(uchar *buf, uint index, const uchar *key,
   ha_statistic_increment(&SSV::ha_read_key_count);
   int error=mi_rkey(file, buf, index, key, keypart_map, find_flag);
   table->status=error ? STATUS_NOT_FOUND: 0;
-  if (!error)
-  {
-    rows_read++;
-
-    int inx = index;
-    if (inx >= 0 && inx < (int)MAX_KEY)
-      index_rows_read[inx]++;
-  }
   MYSQL_INDEX_READ_ROW_DONE(error);
   return error;
 }
@@ -1768,14 +1750,6 @@ int ha_myisam::index_read_last_map(uchar *buf, const uchar *key,
   int error=mi_rkey(file, buf, active_index, key, keypart_map,
                     HA_READ_PREFIX_LAST);
   table->status=error ? STATUS_NOT_FOUND: 0;
-  if (!error)
-  {
-    rows_read++;
-
-    int inx = (active_index == MAX_KEY) ? file->lastinx : active_index;
-    if (inx >= 0 && inx < (int)MAX_KEY)
-      index_rows_read[inx]++;
-  }
   MYSQL_INDEX_READ_ROW_DONE(error);
   DBUG_RETURN(error);
 }
@@ -1787,13 +1761,6 @@ int ha_myisam::index_next(uchar *buf)
   ha_statistic_increment(&SSV::ha_read_next_count);
   int error=mi_rnext(file,buf,active_index);
   table->status=error ? STATUS_NOT_FOUND: 0;
-  if (!error) {
-    rows_read++;
-
-    int inx = (active_index == MAX_KEY) ? file->lastinx : active_index;
-    if (inx >= 0 && inx < (int)MAX_KEY)
-      index_rows_read[inx]++;
-  }
   MYSQL_INDEX_READ_ROW_DONE(error);
   return error;
 }
@@ -1805,13 +1772,6 @@ int ha_myisam::index_prev(uchar *buf)
   ha_statistic_increment(&SSV::ha_read_prev_count);
   int error=mi_rprev(file,buf, active_index);
   table->status=error ? STATUS_NOT_FOUND: 0;
-  if (!error) {
-    rows_read++;
-
-    int inx = (active_index == MAX_KEY) ? file->lastinx : active_index;
-    if (inx >= 0 && inx < (int)MAX_KEY)
-      index_rows_read[inx]++;
-  }
   MYSQL_INDEX_READ_ROW_DONE(error);
   return error;
 }
@@ -1823,14 +1783,6 @@ int ha_myisam::index_first(uchar *buf)
   ha_statistic_increment(&SSV::ha_read_first_count);
   int error=mi_rfirst(file, buf, active_index);
   table->status=error ? STATUS_NOT_FOUND: 0;
-  if (!error)
-  {
-    rows_read++;
-
-    int inx = (active_index == MAX_KEY) ? file->lastinx : active_index;
-    if (inx >= 0 && inx < (int)MAX_KEY)
-      index_rows_read[inx]++;
-  }
   MYSQL_INDEX_READ_ROW_DONE(error);
   return error;
 }
@@ -1842,14 +1794,6 @@ int ha_myisam::index_last(uchar *buf)
   ha_statistic_increment(&SSV::ha_read_last_count);
   int error=mi_rlast(file, buf, active_index);
   table->status=error ? STATUS_NOT_FOUND: 0;
-  if (!error)
-  {
-    rows_read++;
-
-    int inx = (active_index == MAX_KEY) ? file->lastinx : active_index;
-    if (inx >= 0 && inx < (int)MAX_KEY)
-      index_rows_read[inx]++;
-  }
   MYSQL_INDEX_READ_ROW_DONE(error);
   return error;
 }
@@ -1867,14 +1811,6 @@ int ha_myisam::index_next_same(uchar *buf,
     error= mi_rnext_same(file,buf);
   } while (error == HA_ERR_RECORD_DELETED);
   table->status=error ? STATUS_NOT_FOUND: 0;
-  if (!error)
-  {
-    rows_read++;
-
-    int inx = (active_index == MAX_KEY) ? file->lastinx : active_index;
-    if (inx >= 0 && inx < (int)MAX_KEY)
-      index_rows_read[inx]++;
-  }
   MYSQL_INDEX_READ_ROW_DONE(error);
   return error;
 }
@@ -1894,8 +1830,6 @@ int ha_myisam::rnd_next(uchar *buf)
   ha_statistic_increment(&SSV::ha_read_rnd_next_count);
   int error=mi_scan(file, buf);
   table->status=error ? STATUS_NOT_FOUND: 0;
-  if (!error)
-    rows_read++;
   MYSQL_READ_ROW_DONE(error);
   return error;
 }
@@ -1907,8 +1841,6 @@ int ha_myisam::rnd_pos(uchar *buf, uchar *pos)
   ha_statistic_increment(&SSV::ha_read_rnd_count);
   int error=mi_rrnd(file, buf, my_get_ptr(pos,ref_length));
   table->status=error ? STATUS_NOT_FOUND: 0;
-  if (!error)
-    rows_read++;
   MYSQL_READ_ROW_DONE(error);
   return error;
 }
@@ -2372,7 +2304,8 @@ static int myisam_init(void *p)
   myisam_hton->create= myisam_create_handler;
   myisam_hton->panic= myisam_panic;
   myisam_hton->close_connection= myisam_close_connection;
-  myisam_hton->flags= HTON_CAN_RECREATE | HTON_SUPPORT_LOG_TABLES;
+  myisam_hton->flags= HTON_CAN_RECREATE | HTON_SUPPORT_LOG_TABLES |
+                      HTON_SUPPORTS_PACKED_KEYS;
   myisam_hton->is_supported_system_table= myisam_is_supported_system_table;
 
   main_thread_keycache_var= st_keycache_thread_var();
