@@ -1,4 +1,6 @@
 /* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2018, Percona and/or its affiliates. All rights reserved.
+   Copyright (c) 2009, 2016, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,10 +21,9 @@
 */
 
 #include <algorithm>
-#include <functional>
-#include <my_global.h>
+
 #include "fulltext.h"
-#if defined(__WIN__)
+#if defined(_WIN32)
 #include <fcntl.h>
 #else
 #include <stddef.h>
@@ -39,6 +40,7 @@
 #define MYF_RW  MYF(MY_NABP | MY_WME | MY_WAIT_IF_FULL)
 #define DISK_BUFFER_SIZE (IO_SIZE*16)
 
+
 class Key_compare :
   public std::binary_function<const uchar*, const uchar*, bool>
 {
@@ -51,6 +53,7 @@ public:
   MI_SORT_PARAM *info;
 };
 
+
 /*
  Pointers of functions for store and read keys from temp file
 */
@@ -59,42 +62,42 @@ extern void print_error(const char *fmt,...);
 
 /* Functions defined in this file */
 
-static ha_rows find_all_keys(MI_SORT_PARAM *info, ulong keys,
+static ha_rows find_all_keys(MI_SORT_PARAM *info,uint keys,
                              uchar **sort_keys,
-                             DYNAMIC_ARRAY *buffpek, long *maxbuffer,
+                             DYNAMIC_ARRAY *buffpek,int *maxbuffer,
                              IO_CACHE *tempfile,
                              IO_CACHE *tempfile_for_exceptions);
 static int write_keys(MI_SORT_PARAM *info,uchar **sort_keys,
-                      ulong count, BUFFPEK *buffpek,IO_CACHE *tempfile);
+                      uint count, BUFFPEK *buffpek,IO_CACHE *tempfile);
 static int write_key(MI_SORT_PARAM *info, uchar *key,
                      IO_CACHE *tempfile);
 static int write_index(MI_SORT_PARAM *info,uchar * *sort_keys,
-                       ulong count);
-static int merge_many_buff(MI_SORT_PARAM *info, ulong keys,
+                       uint count);
+static int merge_many_buff(MI_SORT_PARAM *info,uint keys,
                            uchar * *sort_keys,
-                           BUFFPEK *buffpek, long *maxbuffer,
+                           BUFFPEK *buffpek,int *maxbuffer,
                            IO_CACHE *t_file);
-static ulong read_to_buffer(IO_CACHE *fromfile,BUFFPEK *buffpek,
+static uint read_to_buffer(IO_CACHE *fromfile,BUFFPEK *buffpek,
                            uint sort_length);
-static int merge_buffers(MI_SORT_PARAM *info, ulong keys,
+static int merge_buffers(MI_SORT_PARAM *info,uint keys,
                          IO_CACHE *from_file, IO_CACHE *to_file,
                          uchar * *sort_keys, BUFFPEK *lastbuff,
                          BUFFPEK *Fb, BUFFPEK *Tb);
-static int merge_index(MI_SORT_PARAM *, ulong, uchar **, BUFFPEK *, long,
+static int merge_index(MI_SORT_PARAM *,uint,uchar **,BUFFPEK *, int,
                        IO_CACHE *);
 static int flush_ft_buf(MI_SORT_PARAM *info);
 
 static int write_keys_varlen(MI_SORT_PARAM *info,uchar **sort_keys,
-                             ulong count, BUFFPEK *buffpek,
+                             uint count, BUFFPEK *buffpek,
                              IO_CACHE *tempfile);
-static ulong read_to_buffer_varlen(IO_CACHE *fromfile,BUFFPEK *buffpek,
-                                   uint sort_length);
+static uint read_to_buffer_varlen(IO_CACHE *fromfile,BUFFPEK *buffpek,
+                                  uint sort_length);
 static int write_merge_key(MI_SORT_PARAM *info, IO_CACHE *to_file,
-                           uchar *key, uint sort_length, ulong count);
+                           uchar *key, uint sort_length, uint count);
 static int write_merge_key_varlen(MI_SORT_PARAM *info,
                                   IO_CACHE *to_file,
                                   uchar* key, uint sort_length,
-                                  ulong count);
+                                  uint count);
 static inline int
 my_var_write(MI_SORT_PARAM *info, IO_CACHE *to_file, uchar *bufs);
 
@@ -115,9 +118,8 @@ my_var_write(MI_SORT_PARAM *info, IO_CACHE *to_file, uchar *bufs);
 int _create_index_by_sort(MI_SORT_PARAM *info,my_bool no_messages,
 			  ulonglong sortbuff_size)
 {
-  int error;
-  long maxbuffer,skr;
-  ulong sort_length, keys;
+  int error,maxbuffer,skr;
+  uint sort_length, keys= 0;
   ulonglong memavl, old_memavl;
   DYNAMIC_ARRAY buffpek;
   ha_rows records;
@@ -148,38 +150,42 @@ int _create_index_by_sort(MI_SORT_PARAM *info,my_bool no_messages,
   memavl= MY_MAX(sortbuff_size, MIN_SORT_BUFFER);
   records=	info->sort_info->max_records;
   sort_length=	info->key_length;
-  LINT_INIT(keys);
 
   if ((memavl - sizeof(BUFFPEK)) / (sort_length + sizeof(char *)) > UINT_MAX32)
     memavl= sizeof(BUFFPEK) + UINT_MAX32 * (sort_length + sizeof(char *));
 
   while (memavl >= MIN_SORT_BUFFER)
   {
-    if ((records < ULONG_MAX) &&
+    if ((records < UINT_MAX32) && 
        ((my_off_t) (records + 1) * 
         (sort_length + sizeof(char*)) <= (my_off_t) memavl))
-      keys= (ulong) records + 1;
+      keys= (uint)records+1;
     else
       do
       {
 	skr=maxbuffer;
-        if (memavl < sizeof(BUFFPEK) * (ulong) maxbuffer ||
-            (keys = (memavl - sizeof(BUFFPEK) * (ulong) maxbuffer) /
-             (sort_length+sizeof(char*))) <= 1 ||
-            keys < (ulong) maxbuffer)
+	if (memavl < sizeof(BUFFPEK)*(uint) maxbuffer ||
+	    (keys= (uint)((memavl-sizeof(BUFFPEK)*(uint) maxbuffer)/
+                          (sort_length+sizeof(char*)))) <= 1 ||
+            keys < (uint) maxbuffer)
 	{
 	  mi_check_print_error(info->sort_info->param,
 			       "myisam_sort_buffer_size is too small");
 	  goto err;
 	}
       }
-      while ((maxbuffer= (long) (records / (keys - 1) + 1)) != skr);
+      while ((maxbuffer= (int) (records/(keys-1)+1)) != skr);
 
-    if ((sort_keys=(uchar **)my_malloc(keys*(sort_length+sizeof(char*))+
+    if ((sort_keys=(uchar **)my_malloc(PSI_NOT_INSTRUMENTED,
+                                       keys*(sort_length+sizeof(char*))+
 				       HA_FT_MAXBYTELEN, MYF(0))))
     {
-      if (my_init_dynamic_array(&buffpek, sizeof(BUFFPEK), maxbuffer,
-			     maxbuffer/2))
+      if (my_init_dynamic_array(&buffpek,
+                                PSI_NOT_INSTRUMENTED,
+                                sizeof(BUFFPEK),
+                                NULL,
+                                maxbuffer,
+                                maxbuffer/2))
       {
 	my_free(sort_keys);
         sort_keys= 0;
@@ -199,7 +205,7 @@ int _create_index_by_sort(MI_SORT_PARAM *info,my_bool no_messages,
   (*info->lock_in_memory)(info->sort_info->param);/* Everything is allocated */
 
   if (!no_messages)
-    printf("  - Searching for keys, allocating buffer for %lu keys\n", keys);
+    printf("  - Searching for keys, allocating buffer for %d keys\n",keys);
 
   if ((records=find_all_keys(info,keys,sort_keys,&buffpek,&maxbuffer,
                                   &tempfile,&tempfile_for_exceptions))
@@ -209,7 +215,7 @@ int _create_index_by_sort(MI_SORT_PARAM *info,my_bool no_messages,
   {
     if (!no_messages)
       printf("  - Dumping %lu keys\n", (ulong) records);
-    if (write_index(info,sort_keys, (ulong) records))
+    if (write_index(info,sort_keys, (uint) records))
       goto err; /* purecov: inspected */
   }
   else
@@ -272,13 +278,13 @@ err:
 
 /* Search after all keys and place them in a temp. file */
 
-static ha_rows find_all_keys(MI_SORT_PARAM *info, ulong keys,
+static ha_rows find_all_keys(MI_SORT_PARAM *info, uint keys,
                              uchar **sort_keys, DYNAMIC_ARRAY *buffpek,
-                             long *maxbuffer, IO_CACHE *tempfile,
+                             int *maxbuffer, IO_CACHE *tempfile,
                              IO_CACHE *tempfile_for_exceptions)
 {
   int error;
-  ulong idx;
+  uint idx;
   DBUG_ENTER("find_all_keys");
 
   idx=error=0;
@@ -323,22 +329,26 @@ static ha_rows find_all_keys(MI_SORT_PARAM *info, ulong keys,
 
 /* Search after all keys and place them in a temp. file */
 
-pthread_handler_t thr_find_all_keys(void *arg)
+void *thr_find_all_keys(void *arg)
 {
   MI_SORT_PARAM *sort_param= (MI_SORT_PARAM*) arg;
   int error;
   ulonglong memavl, old_memavl;
-  ulong keys, sort_length;
-  ulong idx, maxbuffer;
+  uint keys= 0, sort_length;
+  uint idx, maxbuffer;
   uchar **sort_keys=0;
-
-  LINT_INIT(keys);
+  st_keycache_thread_var thread_keycache_var;
 
   error=1;
 
   if (my_thread_init())
     goto err;
 
+  memset(&thread_keycache_var, 0, sizeof(st_keycache_thread_var));
+  mysql_cond_init(PSI_NOT_INSTRUMENTED,
+                  &thread_keycache_var.suspend);
+  my_set_thread_local(keycache_tls_key, &thread_keycache_var);
+
   { /* Add extra block since DBUG_ENTER declare variables */
     DBUG_ENTER("thr_find_all_keys");
     DBUG_PRINT("enter", ("master: %d", sort_param->master));
@@ -365,7 +375,7 @@ pthread_handler_t thr_find_all_keys(void *arg)
     sort_keys= (uchar **) NULL;
 
     memavl=       MY_MAX(sort_param->sortbuff_size, MIN_SORT_BUFFER);
-    idx=          (ulong) sort_param->sort_info->max_records;
+    idx=          (uint)sort_param->sort_info->max_records;
     sort_length=  sort_param->key_length;
     maxbuffer=    1;
 
@@ -380,28 +390,32 @@ pthread_handler_t thr_find_all_keys(void *arg)
         keys= idx+1;
       else
       {
-        ulong skr;
+        uint skr;
         do
         {
           skr= maxbuffer;
           if (memavl < sizeof(BUFFPEK)*maxbuffer ||
-              (keys=(memavl-sizeof(BUFFPEK)*maxbuffer)/
-               (sort_length+sizeof(char*))) <= 1 ||
-              keys < maxbuffer)
+              (keys=(uint)((memavl-sizeof(BUFFPEK)*maxbuffer)/
+                           (sort_length+sizeof(char*)))) <= 1 ||
+              keys < (uint) maxbuffer)
           {
             mi_check_print_error(sort_param->sort_info->param,
                                  "myisam_sort_buffer_size is too small");
             goto err;
           }
         }
-        while ((maxbuffer= (idx/(keys-1)+1)) != skr);
+        while ((maxbuffer= (int) (idx/(keys-1)+1)) != skr);
       }
       if ((sort_keys= (uchar**)
-           my_malloc(keys*(sort_length+sizeof(char*))+
+           my_malloc(PSI_NOT_INSTRUMENTED,
+                     keys*(sort_length+sizeof(char*))+
                      ((sort_param->keyinfo->flag & HA_FULLTEXT) ?
                       HA_FT_MAXBYTELEN : 0), MYF(0))))
       {
-        if (my_init_dynamic_array(&sort_param->buffpek, sizeof(BUFFPEK),
+        if (my_init_dynamic_array(&sort_param->buffpek,
+                                  PSI_NOT_INSTRUMENTED,
+                                  sizeof(BUFFPEK),
+                                  NULL,
                                   maxbuffer, maxbuffer/2))
         {
           my_free(sort_keys);
@@ -423,7 +437,7 @@ pthread_handler_t thr_find_all_keys(void *arg)
     }
 
     if (sort_param->sort_info->param->testflag & T_VERBOSE)
-      printf("Key %d - Allocating buffer for %lu keys\n",
+      printf("Key %d - Allocating buffer for %d keys\n",
              sort_param->key + 1, keys);
     sort_param->sort_keys= sort_keys;
 
@@ -503,6 +517,7 @@ ok:
     mysql_mutex_unlock(&sort_param->sort_info->mutex);
     DBUG_PRINT("exit", ("======== ending thread ========"));
   }
+  mysql_cond_destroy(&thread_keycache_var.suspend);
   my_thread_end();
   return NULL;
 }
@@ -512,7 +527,8 @@ int thr_write_keys(MI_SORT_PARAM *sort_param)
 {
   SORT_INFO *sort_info=sort_param->sort_info;
   MI_CHECK *param=sort_info->param;
-  ulong UNINIT_VAR(length), keys;
+  size_t length= 0;
+  ulong keys;
   ulong *rec_per_key_part=param->rec_per_key_part;
   int got_error=sort_info->got_error;
   uint i;
@@ -521,7 +537,6 @@ int thr_write_keys(MI_SORT_PARAM *sort_param)
   MI_SORT_PARAM *sinfo;
   uchar *mergebuf=0;
   DBUG_ENTER("thr_write_keys");
-  LINT_INIT(length);
 
   for (i= 0, sinfo= sort_param ;
        i < sort_info->total_keys ;
@@ -577,13 +592,14 @@ int thr_write_keys(MI_SORT_PARAM *sort_param)
     }
     if (sinfo->buffpek.elements)
     {
-      ulong maxbuffer=sinfo->buffpek.elements-1;
+      uint maxbuffer=sinfo->buffpek.elements-1;
       if (!mergebuf)
       {
-        length=param->sort_buffer_length;
+        length= (size_t)param->sort_buffer_length;
         while (length >= MIN_SORT_BUFFER)
         {
-          if ((mergebuf= (uchar *) my_malloc(length, MYF(0))))
+          if ((mergebuf= (uchar *) my_malloc(PSI_NOT_INSTRUMENTED,
+                                             length, MYF(0))))
               break;
           length=length*3/4;
         }
@@ -600,7 +616,7 @@ int thr_write_keys(MI_SORT_PARAM *sort_param)
           printf("Key %d  - Merging %u keys\n",sinfo->key+1, sinfo->keys);
         if (merge_many_buff(sinfo, keys, (uchar **)mergebuf,
 			    dynamic_element(&sinfo->buffpek, 0, BUFFPEK *),
-			    (long *) &maxbuffer, &sinfo->tempfile))
+			    (int*) &maxbuffer, &sinfo->tempfile))
         {
           got_error=1;
           continue;
@@ -664,7 +680,7 @@ int thr_write_keys(MI_SORT_PARAM *sort_param)
         /* Write all keys in memory to file for later merge */
 
 static int write_keys(MI_SORT_PARAM *info, uchar **sort_keys,
-                      ulong count, BUFFPEK *buffpek, IO_CACHE *tempfile)
+                      uint count, BUFFPEK *buffpek, IO_CACHE *tempfile)
 {
   uchar **end;
   uint sort_length=info->key_length;
@@ -706,7 +722,7 @@ my_var_write(MI_SORT_PARAM *info, IO_CACHE *to_file, uchar *bufs)
 
 static int write_keys_varlen(MI_SORT_PARAM *info,
                              uchar **sort_keys,
-                             ulong count, BUFFPEK *buffpek,
+                             uint count, BUFFPEK *buffpek,
                              IO_CACHE *tempfile)
 {
   uchar **end;
@@ -751,7 +767,7 @@ static int write_key(MI_SORT_PARAM *info, uchar *key, IO_CACHE *tempfile)
 /* Write index */
 
 static int write_index(MI_SORT_PARAM *info, uchar **sort_keys,
-                       ulong count)
+                       uint count)
 {
   DBUG_ENTER("write_index");
 
@@ -768,11 +784,11 @@ static int write_index(MI_SORT_PARAM *info, uchar **sort_keys,
 
         /* Merge buffers to make < MERGEBUFF2 buffers */
 
-static int merge_many_buff(MI_SORT_PARAM *info, ulong keys,
+static int merge_many_buff(MI_SORT_PARAM *info, uint keys,
                            uchar **sort_keys, BUFFPEK *buffpek,
-                           long *maxbuffer, IO_CACHE *t_file)
+                           int *maxbuffer, IO_CACHE *t_file)
 {
-  long i;
+  int i;
   IO_CACHE t_file2, *from_file, *to_file, *temp;
   BUFFPEK *lastbuff;
   DBUG_ENTER("merge_many_buff");
@@ -787,8 +803,10 @@ static int merge_many_buff(MI_SORT_PARAM *info, ulong keys,
   from_file= t_file ; to_file= &t_file2;
   while (*maxbuffer >= MERGEBUFF2)
   {
-    reinit_io_cache(from_file,READ_CACHE,0L,0,0);
-    reinit_io_cache(to_file,WRITE_CACHE,0L,0,0);
+    if (reinit_io_cache(from_file, READ_CACHE, 0L, 0, 0))
+      goto cleanup;
+    if (reinit_io_cache(to_file, WRITE_CACHE, 0L, 0, 0))
+      goto cleanup;
     lastbuff=buffpek;
     for (i=0 ; i <= *maxbuffer-MERGEBUFF*3/2 ; i+=MERGEBUFF)
     {
@@ -802,7 +820,7 @@ static int merge_many_buff(MI_SORT_PARAM *info, ulong keys,
     if (flush_io_cache(to_file))
       break;                                    /* purecov: inspected */
     temp=from_file; from_file=to_file; to_file=temp;
-    *maxbuffer= (long) (lastbuff-buffpek)-1;
+    *maxbuffer= (int) (lastbuff-buffpek)-1;
   }
 cleanup:
   close_cached_file(to_file);                   /* This holds old result */
@@ -831,18 +849,17 @@ cleanup:
     -1	Error
 */
 
-static ulong read_to_buffer(IO_CACHE *fromfile, BUFFPEK *buffpek,
-                            uint sort_length)
+static uint read_to_buffer(IO_CACHE *fromfile, BUFFPEK *buffpek,
+                           uint sort_length)
 {
-  ulong count;
-  ulong length;
+  uint count;
+  uint length;
 
-  if ((count=(ulong) MY_MIN((ha_rows) buffpek->max_keys,buffpek->count)))
+  if ((count=(uint) MY_MIN((ha_rows) buffpek->max_keys,buffpek->count)))
   {
-    if (mysql_file_pread(fromfile->file, (uchar*) buffpek->base,
-                         (length= sort_length*count),
-                         buffpek->file_pos, MYF_RW))
-      return((ulong) -1);                        /* purecov: inspected */
+    if (my_b_pread(fromfile, buffpek->base,
+                   (length= sort_length * count), buffpek->file_pos))
+      return((uint) -1);                        /* purecov: inspected */
     buffpek->key=buffpek->base;
     buffpek->file_pos+= length;                 /* New filepos */
     buffpek->count-=    count;
@@ -851,27 +868,27 @@ static ulong read_to_buffer(IO_CACHE *fromfile, BUFFPEK *buffpek,
   return (count*sort_length);
 } /* read_to_buffer */
 
-static ulong read_to_buffer_varlen(IO_CACHE *fromfile, BUFFPEK *buffpek,
+static uint read_to_buffer_varlen(IO_CACHE *fromfile, BUFFPEK *buffpek,
                                   uint sort_length)
 {
-  ulong count;
+  uint count;
   uint16 length_of_key = 0;
-  ulong idx;
+  uint idx;
   uchar *buffp;
 
-  if ((count=(ulong) MY_MIN((ha_rows) buffpek->max_keys, buffpek->count)))
+  if ((count=(uint) MY_MIN((ha_rows) buffpek->max_keys, buffpek->count)))
   {
     buffp = buffpek->base;
 
     for (idx=1;idx<=count;idx++)
     {
-      if (mysql_file_pread(fromfile->file, (uchar*)&length_of_key,
-                           sizeof(length_of_key), buffpek->file_pos, MYF_RW))
-        return((ulong) -1);
+      if (my_b_pread(fromfile, (uchar*)&length_of_key,
+                     sizeof(length_of_key), buffpek->file_pos))
+        return((uint) -1);
       buffpek->file_pos+=sizeof(length_of_key);
-      if (mysql_file_pread(fromfile->file, (uchar*) buffp,
-                           length_of_key, buffpek->file_pos, MYF_RW))
-        return((ulong) -1);
+      if (my_b_pread(fromfile, buffp, length_of_key,
+                     buffpek->file_pos))
+        return((uint) -1);
       buffpek->file_pos+=length_of_key;
       buffp = buffp + sort_length;
     }
@@ -885,9 +902,9 @@ static ulong read_to_buffer_varlen(IO_CACHE *fromfile, BUFFPEK *buffpek,
 
 static int write_merge_key_varlen(MI_SORT_PARAM *info,
                                   IO_CACHE *to_file, uchar* key,
-                                  uint sort_length, ulong count)
+                                  uint sort_length, uint count)
 {
-  ulong idx;
+  uint idx;
   uchar *bufs = key;
 
   for (idx=1;idx<=count;idx++)
@@ -903,7 +920,7 @@ static int write_merge_key_varlen(MI_SORT_PARAM *info,
 
 static int write_merge_key(MI_SORT_PARAM *info MY_ATTRIBUTE((unused)),
                            IO_CACHE *to_file, uchar *key,
-                           uint sort_length, ulong count)
+                           uint sort_length, uint count)
 {
   return my_b_write(to_file, key, (size_t) sort_length*count);
 }
@@ -914,15 +931,14 @@ static int write_merge_key(MI_SORT_PARAM *info MY_ATTRIBUTE((unused)),
 */
 
 static int
-merge_buffers(MI_SORT_PARAM *info, ulong keys, IO_CACHE *from_file,
+merge_buffers(MI_SORT_PARAM *info, uint keys, IO_CACHE *from_file,
               IO_CACHE *to_file, uchar **sort_keys, BUFFPEK *lastbuff,
               BUFFPEK *Fb, BUFFPEK *Tb)
 {
-  ulong error;
-  uint sort_length;
-  ulong maxcount;
+  int error;
+  uint sort_length,maxcount;
   ha_rows count;
-  my_off_t UNINIT_VAR(to_start_filepos);
+  my_off_t to_start_filepos= 0;
   uchar *strpos;
   BUFFPEK *buffpek,**refpek;
   QUEUE queue;
@@ -930,15 +946,14 @@ merge_buffers(MI_SORT_PARAM *info, ulong keys, IO_CACHE *from_file,
   DBUG_ENTER("merge_buffers");
 
   count=error=0;
-  maxcount= keys / ((ulong) (Tb-Fb) + 1);
+  maxcount=keys/((uint) (Tb-Fb) +1);
   DBUG_ASSERT(maxcount > 0);
-  LINT_INIT(to_start_filepos);
   if (to_file)
     to_start_filepos=my_b_tell(to_file);
   strpos=(uchar*) sort_keys;
   sort_length=info->key_length;
 
-  if (init_queue(&queue, (uint) (Tb-Fb)+1, offsetof(BUFFPEK,key), 0,
+  if (init_queue(&queue,(uint) (Tb-Fb)+1,offsetof(BUFFPEK,key),0,
                  (int (*)(void*, uchar *,uchar*)) info->key_cmp,
                  (void*) info))
     DBUG_RETURN(1); /* purecov: inspected */
@@ -948,8 +963,9 @@ merge_buffers(MI_SORT_PARAM *info, ulong keys, IO_CACHE *from_file,
     count+= buffpek->count;
     buffpek->base= strpos;
     buffpek->max_keys=maxcount;
-    strpos+= (error= info->read_to_buffer(from_file,buffpek, sort_length));
-    if (error == (ulong) -1)
+    strpos+= (uint) (error=(int) info->read_to_buffer(from_file,buffpek,
+                                                      sort_length));
+    if (error == -1)
       goto err; /* purecov: inspected */
     queue_insert(&queue,(uchar*) buffpek);
   }
@@ -981,10 +997,10 @@ merge_buffers(MI_SORT_PARAM *info, ulong keys, IO_CACHE *from_file,
       buffpek->key+=sort_length;
       if (! --buffpek->mem_count)
       {
-        if (!(error= info->read_to_buffer(from_file,buffpek,sort_length)))
+        if (!(error=(int) info->read_to_buffer(from_file,buffpek,sort_length)))
         {
           uchar *base=buffpek->base;
-          ulong max_keys=buffpek->max_keys;
+          uint max_keys=buffpek->max_keys;
 
           (void) queue_remove(&queue,0);
 
@@ -1009,7 +1025,7 @@ merge_buffers(MI_SORT_PARAM *info, ulong keys, IO_CACHE *from_file,
           break;                /* One buffer have been removed */
         }
       }
-      else if (error == (ulong) -1)
+      else if (error == -1)
         goto err;               /* purecov: inspected */
       queue_replaced(&queue);   /* Top element has been replaced */
     }
@@ -1042,23 +1058,23 @@ merge_buffers(MI_SORT_PARAM *info, ulong keys, IO_CACHE *from_file,
       }
     }
   }
-  while ((error= info->read_to_buffer(from_file,buffpek,sort_length))
-         != (ulong) -1 && error != 0);
+  while ((error=(int) info->read_to_buffer(from_file,buffpek,sort_length)) != -1 &&
+         error != 0);
 
   lastbuff->count=count;
   if (to_file)
     lastbuff->file_pos=to_start_filepos;
 err:
   delete_queue(&queue);
-  DBUG_RETURN(error != 0);
+  DBUG_RETURN(error);
 } /* merge_buffers */
 
 
         /* Do a merge to output-file (save only positions) */
 
 static int
-merge_index(MI_SORT_PARAM *info, ulong keys, uchar **sort_keys,
-            BUFFPEK *buffpek, long maxbuffer, IO_CACHE *tempfile)
+merge_index(MI_SORT_PARAM *info, uint keys, uchar **sort_keys,
+            BUFFPEK *buffpek, int maxbuffer, IO_CACHE *tempfile)
 {
   DBUG_ENTER("merge_index");
   if (merge_buffers(info,keys,tempfile,(IO_CACHE*) 0,sort_keys,buffpek,buffpek,
