@@ -1,20 +1,13 @@
-/* Copyright (c) 2012, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License, version 2.0,
-   as published by the Free Software Foundation.
-
-   This program is also distributed with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
-   as designated in a particular file or component or in included license
-   documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License, version 2.0, for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
@@ -27,6 +20,7 @@
 
 #include "table_cache.h"
 
+#include "mysqld_thd_manager.h"
 #include "ha_example.h"
 
 /*
@@ -39,6 +33,10 @@ namespace table_cache_unittest {
 
 using my_testing::Server_initializer;
 
+#ifdef SAFE_MUTEX
+static const char *assert_string=
+  ".*Assertion.*count > 0.*my_thread_equal.*";
+#endif
 
 /**
   Test fixture for basic tests involving Table_cache
@@ -56,10 +54,13 @@ protected:
 
   virtual void SetUp()
   {
+    Global_THD_manager *thd_manager= Global_THD_manager::get_instance();
+    thd_manager->set_unit_test();
+    // Reset thread ID counter for each test.
+    thd_manager->set_thread_id_counter(1);
     for (uint i= 0; i < MAX_THREADS; ++i)
     {
       initializer[i].SetUp();
-      initializer[i].thd()->thread_id= i + 1;
     }
 
     ::testing::FLAGS_gtest_death_test_style = "threadsafe";
@@ -132,7 +133,6 @@ class Mock_share : public TABLE_SHARE
 public:
   Mock_share(const char *key)
   {
-    memset(static_cast<void*>(this), 0, sizeof(TABLE_SHARE));
     /*
       Both table_cache_key and cache_element array are used by
       Table_cache code.
@@ -142,7 +142,7 @@ public:
     memset(cache_element_arr, 0, sizeof(cache_element_arr));
     cache_element= cache_element_arr;
     // MEM_ROOT is used for constructing ha_example() instances.
-    init_alloc_root(&m_mem_root, 1024, 0);
+    init_alloc_root(PSI_NOT_INSTRUMENTED, &m_mem_root, 1024, 0);
     /*
       Assertion in some of Table_cache methods check that version of
       the share is up-to-date.
@@ -159,9 +159,9 @@ public:
 
   TABLE *create_table(THD *thd)
   {
-    TABLE *result= (TABLE *)my_malloc(sizeof(TABLE), MYF(0));
+    TABLE *result= (TABLE *)my_malloc(PSI_NOT_INSTRUMENTED, sizeof(TABLE), MYF(0));
 
-    memset(static_cast<void*>(result), 0, sizeof(TABLE));
+    ::new(result) TABLE;
     result->s= this;
     // We create TABLE which is already marked as used
     result->in_use= thd;
@@ -202,7 +202,7 @@ TEST_F(TableCacheBasicDeathTest, CacheCreateAndDestroy)
   // Cache should be not locked after creation
 #ifdef SAFE_MUTEX
   MY_EXPECT_DEATH_IF_SUPPORTED(table_cache.assert_owner(),
-                            ".*Assertion.*count > 0.*pthread_equal.*");
+                            assert_string);
 #endif
   table_cache.destroy();
 }
@@ -221,7 +221,7 @@ TEST_F(TableCacheBasicDeathTest, CacheLockAndUnlock)
 #ifdef SAFE_MUTEX
   // Cache should not be locked after creation
   MY_EXPECT_DEATH_IF_SUPPORTED(table_cache.assert_owner(),
-                            ".*Assertion.*count > 0.*pthread_equal.*");
+                            assert_string);
 #endif
 
   // And get locked after we call its lock() method
@@ -232,7 +232,7 @@ TEST_F(TableCacheBasicDeathTest, CacheLockAndUnlock)
   table_cache.unlock();
 #ifdef SAFE_MUTEX
   MY_EXPECT_DEATH_IF_SUPPORTED(table_cache.assert_owner(),
-                            ".*Assertion.*count > 0.*pthread_equal.*");
+                            assert_string);
 #endif
 
   table_cache.destroy();
@@ -273,9 +273,9 @@ TEST_F(TableCacheBasicDeathTest, ManagerCreateAndDestroy)
   // And not locked
 #ifdef SAFE_MUTEX
   MY_EXPECT_DEATH_IF_SUPPORTED(cache_1->assert_owner(),
-                            ".*Assertion.*count > 0.*pthread_equal.*");
+                            assert_string);
   MY_EXPECT_DEATH_IF_SUPPORTED(cache_2->assert_owner(),
-                            ".*Assertion.*count > 0.*pthread_equal.*");
+                            assert_string);
 #endif
 
   table_cache_manager.destroy();
@@ -744,9 +744,9 @@ TEST_F(TableCacheDoubleCacheDeathTest, ManagerLockAndUnlock)
   // Nor caches nor LOCK_open should not be locked after initialization
 #ifdef SAFE_MUTEX
   MY_EXPECT_DEATH_IF_SUPPORTED(table_cache_manager.assert_owner_all(),
-                            ".*Assertion.*count > 0.*pthread_equal.*");
+                            assert_string);
   MY_EXPECT_DEATH_IF_SUPPORTED(table_cache_manager.assert_owner_all_and_tdc(),
-                            ".*Assertion.*count > 0.*pthread_equal.*");
+                            assert_string);
 #endif
 
   // And get locked after we call its lock_all_and_tdc() method.
@@ -768,9 +768,9 @@ TEST_F(TableCacheDoubleCacheDeathTest, ManagerLockAndUnlock)
 
 #ifdef SAFE_MUTEX
   MY_EXPECT_DEATH_IF_SUPPORTED(table_cache_manager.assert_owner_all(),
-                            ".*Assertion.*count > 0.*pthread_equal.*");
+                            assert_string);
   MY_EXPECT_DEATH_IF_SUPPORTED(table_cache_manager.assert_owner_all_and_tdc(),
-                            ".*Assertion.*count > 0.*pthread_equal.*");
+                            assert_string);
 #endif
 }
 
